from __future__ import annotations

from tcsoa.gen.BusinessObjects import RequestObject, AppInterface
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Ai._2006_03.Ai import StatusInfo
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FindRequestsFilter(TcBaseObj):
    """
    structure that captures the filtering options for getting the Request Objects. This will include the AppInterface
    filter options too.
    
    :var aiQryParams: structure to capture the filter options on parent(s) ApplicationInterfaces of the RequestObject(s)
    :var requestState: vector  representing the requeststates to use for filtering. Currently, the only valid values
    are unique combinations of (case sensitive): Processing, Pending, Communicating, Completed, Rejected
    :var requestStatus: vector of strings representing the statuses on the request to search for. Currently, the valid
    values are a combination of (case sensitive): Normal,  Warning, Severe, Abort
    :var stateDescription: state description to use for searching for RequestObject.
    :var statusDescription: status message by which to filter for RequestObjects.
    :var customStrings: vector of strings that have the custom key and value pair to search on.
    :var requestType: the type of request - valid values:Publish, Sync. Empty vector if this filter option is not
    needed.
    """
    aiQryParams: ProjectFilter = None
    requestState: List[str] = ()
    requestStatus: List[str] = ()
    stateDescription: str = ''
    statusDescription: str = ''
    customStrings: StrToStrMap = None
    requestType: List[str] = ()


@dataclass
class FindRequestsResponse(TcBaseObj):
    """
    structure containing the response from findRequests method. partial failures are captured in the serviceData and
    found Requests are captured in RequestDetails.
    
    :var requests: list of found requests based on the input query.
    :var serviceData: Any partial errors are reported in this member.
    """
    requests: List[RequestDetail] = ()
    serviceData: ServiceData = None


@dataclass
class GetProjectsInfo2Response(TcBaseObj):
    """
    get the ProjectInfo structure for each of the specified ApplicationInterfaceObjects.
    
    :var infos: info about the ApplicationInterface objects
    :var serviceData: partial errors are returned in this structure.
    """
    infos: List[ProjectInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetRequestsInfo2Response(TcBaseObj):
    """
    get the details on the input RequestObject.
    
    :var details: details about the input RequestObjects. If there is an error getting info for any RequestObject, it
    will not be in this vector, but failure details will be in the serviceData.
    :var serviceData: serviceData to capture any partialErrors. client id used will be the position in the input vector.
    """
    details: List[RequestDetail] = ()
    serviceData: ServiceData = None


@dataclass
class ProjectFilter(TcBaseObj):
    """
    Structure to specify the filter when using getProjects  method.
    
    :var name: name of the AppliationInterface Object
    :var description: description of the ApplicationInterface Object
    :var releasedAfter: filtering by Date
    :var contextString: maps to the context string of AI object's Export TransferMode.
    :var targetSiteIds: valid site names to be used to search for ApplicationInterface objects.
    :var targetAppProjectId: if an application stamps a targetAppProject id using the setProjectsInfo method - they can
    use this for filtering.
    :var type: type of the ApplicationInterface Object. The type must be a valid type of AI Object.
    :var userId: userId to filter on (maps to owning user)
    :var groupName: filter on AI objects using groupName
    :var createdBefore: filtering by Date
    :var createdAfter: filtering by Date
    :var modifiedBefore: filtering by Date
    :var modifiedAfter: filtering by Date
    :var releasedBefore: filtering by Date
    """
    name: str = ''
    description: str = ''
    releasedAfter: datetime = None
    contextString: str = ''
    targetSiteIds: List[str] = ()
    targetAppProjectId: str = ''
    type: str = ''
    userId: str = ''
    groupName: str = ''
    createdBefore: datetime = None
    createdAfter: datetime = None
    modifiedBefore: datetime = None
    modifiedAfter: datetime = None
    releasedBefore: datetime = None


@dataclass
class ProjectInfo(TcBaseObj):
    """
    Structure to specify ApplicationInterface information.
    
    :var ai: The ApplicationInterfaceObject for which the info is relevant.
    :var targetAppProjectId: The projectId string recorded on the ApplicationInterface Object
    :var name: name of the AppliationInterface Object
    :var description: description of the ApplicationInterface Object
    :var targetSiteIds: The list of names of targetSiteIds to be set on the ApplicationInterface Object. Entries must
    be valid site names.
    """
    ai: AppInterface = None
    targetAppProjectId: str = ''
    name: str = ''
    description: str = ''
    targetSiteIds: List[str] = ()


@dataclass
class RequestDetail(TcBaseObj):
    """
    Structure representing the details of the RequestObject
    
    :var ro: the request object for which the details are being provided.
    :var name: name of the RequestObject
    :var description: description of the RequestObject
    :var stateDesc: description on the state of the RequestObject.
    :var status: the status fields of the RequestObject
    :var rscope: 2 values are possible:  0(whole)- no ExternalReference elements will be found in plmxml. Partial(1)
    then there will be ExternalReference elements in plmxml.
    :var rupdate: used to specify an incremental update. possible values are 0 (Full), 1 ( delta).
    :var kvPairs: key value pairs associated with the RequestObject. These would have been populated via the
    setRequestsInfo call.
    """
    ro: RequestObject = None
    name: str = ''
    description: str = ''
    stateDesc: str = ''
    status: StatusInfo = None
    rscope: int = 0
    rupdate: int = 0
    kvPairs: StrToStrMap = None


@dataclass
class RequestInfo(TcBaseObj):
    """
    structure representing gettable/settable information on a RequestObject by clients for a given RequestObject.
    
    :var ro: the object for which the data is being or retrieved.
    :var stateMessage: The string to set/get for the RequestObject's state_msg property.
    :var statusInfo: Structure to set the status and status_msg attributes of RequestObject. These typically are used
    to provide the TC user information about what happened in the relevant client application.
    :var kvPairs: key value pairs of strings to allow clients to set custom properties on the RequestObject. Limit of
    64 bytes on key and 256 bytes on value.
    """
    ro: RequestObject = None
    stateMessage: str = ''
    statusInfo: StatusInfo = None
    kvPairs: StrToStrMap = None


"""
string to string map.
"""
StrToStrMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.Core._2018_11.ProjectLevelSecurity import ChangeOwningProgramInput2, UserGroupRoleInfo, UserProjectsResponse
from tcsoa.gen.Core._2018_11.LogicalObject import GetLogicalObjectResponse3, GetLogicalObjectInput3
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ProjectLevelSecurityService(TcService):

    @classmethod
    def getUserProjects2(cls, userInfoList: List[UserGroupRoleInfo], activeProjectsOnly: bool, visibleProjectsOnly: bool, privilegedProjectsOnly: bool, programsOnly: bool, programsAndTheChildProjects: bool) -> UserProjectsResponse:
        """
        This operation returns the list of TC_Project objects for the user, group, and role based on the additional
        criteria like active projects only, user privileged projects only and programs only.
        
        Use cases:
        Use Case 1: Changing group/role selection in the user setting dialog. The available projects needs to be
        populated.
        Use Case 2: Assigning workspace objects to project, the assignable projects needs to be populated. If the "show
        all user projects" checkbox is checked, then the group/role of the user session should not be considered.
        """
        return cls.execute_soa_method(
            method_name='getUserProjects2',
            library='Core',
            service_date='2018_11',
            service_name='ProjectLevelSecurity',
            params={'userInfoList': userInfoList, 'activeProjectsOnly': activeProjectsOnly, 'visibleProjectsOnly': visibleProjectsOnly, 'privilegedProjectsOnly': privilegedProjectsOnly, 'programsOnly': programsOnly, 'programsAndTheChildProjects': programsAndTheChildProjects},
            response_cls=UserProjectsResponse,
        )

    @classmethod
    def changeOwningProgram(cls, chgOwnProgramInput: List[ChangeOwningProgramInput2]) -> ServiceData:
        """
        This operation changes the owning program of the given set of objects. Owning Program (owning_project attribute
        ) is changed to the new value passed in.
        """
        return cls.execute_soa_method(
            method_name='changeOwningProgram',
            library='Core',
            service_date='2018_11',
            service_name='ProjectLevelSecurity',
            params={'chgOwnProgramInput': chgOwnProgramInput},
            response_cls=ServiceData,
        )


class LogicalObjectService(TcService):

    @classmethod
    def getLogicalObjectsWithContext(cls, loInputs: List[GetLogicalObjectInput3]) -> GetLogicalObjectResponse3:
        """
        This operation returns the logical object instances for the input list of client id and root object instance
        pairs, logical object type names and a map containing the included logical object ID, or member ID and
        configuration context UID. This operation can also return classification objects [A classification object is
        also called an ICO], if the root object or a member object or an included logical object on the logical object
        type is configured for retrieving classification data. For such use cases it also returns ICO property data in
        an ICO specific property structure. This operation can also return the presented properties of logical object
        instances and  configured included logical object instances. An included logical object is a logical object
        that has been added to the other logical object defintion.
        
        This operation will have a one-to-one mapping between the input list  and output list.  The order in the input
        list is also matched to the order in the output list. In an error scenario, an empty entry in the output list
        will be returned.
        
        Use cases:
        This operation is invoked to retrieve logical object instances by the client by passing a list of input
        structures each containing:
        1.    A list of client id and root object instance pairs.
        2.    A logical object type name.
        3.    A map containing the included logical object ID, or member ID and configuration context UID.
        
        For example, consider the below use case data for the use case(s) specified: 
        
        Use Case Data:
        
        1.    Logical Object definition 1
        Name                                        :     "LogicalObject1"
        Root business object                :       Item
        Included logical object    ID1    :    "LogicalObjectDomainA"
         
        2.    Logical Object definition 2
        Name                                        :     "LogicalObject2"
        Root business object                :       ItemRevision
        Included logical object ID2    :     "LogicalObjectDomainB"
        Included logical object ID3    :     "LogicalObjectDomainC"
        
        3.    Logical Object definition 3
        Name                                        :     "LogicalObject3"
        Root business object                :       Item
        
        4.    Logical Object definition 4
        Name                                        :     "LogicalObject4"
        Root business object                :      ItemRevision
        Retrieve classification data    :      True
        Included logical object ID4    :    "LogicalObjectDomainD"
         
        5.    Logical Object definition 5
        Name                                        :     "LogicalObjectDomainA"
        Root business object                :      ItemRevision
        
        6.    Logical Object definition 6
        Name                                        :     "LogicalObjectDomainB"
        Root business object                :      ItemRevision
        
        7.    Logical Object definition 7
        Name                                        :     "LogicalObjectDomainC"
        Root business object                :      ItemRevision
        
        8.    Logical Object definition 8
        Name                                        :     "LogicalObjectDomainD"
        Root business object                :      ItemRevision
        Retrieve classification data    :      True
        
         9.    Logical object definition 9
        Name            :     "LogicalObject9"
        Root business object    :      Item
        Member ID            :      "Member Revision"
        
        Use Case 1:
         
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {    
                rootObject         =    [Item],
                clientID            =     ["0"]
            }
        loTypeName                     =         "LogicalObject1",   
        loQueryNameValues          =           [<"Included logical object ID1", "ccUid1">]
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs list of size = 1
        and element = 1 contains 1 instance(s) of "LogicalObject1". 
        
        One instance of the included logical object: "LogicalObjectDomainA" will be returned if the root ItemRevision
        object of the included logical object satisfies the configuration context specified by "ccUid1".
        
        The presented properties on the instances of "LogicalObject1" and its included logical object:
        "LogicalObjectDomainA" will be returned through Service Data.
        
        Use Case 2:
        
        GetLogicalObjectInput3[0] = {
            RootObject[0]= { 
                rootObject         =     [Item] , 
                clientID            =     ["0"]
            }
        loTypeName                   =             "LogicalObject2",
        loQueryNameValues    =           [<"Included logical object ID2", "ccUid2">]
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs list of size = 1
        and element = 1 contains 0 instance(s).
        
        Error 39040 - "TYPE_lo_instance_not_found" is stored in the partial error list of service data with "Logical
        Object type name +  root object client id" as the key which helps the caller identify the logical object type
        and root combination for which the logical object instance is not found.
        
        Use Case 3:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {    
                rootObject         =     [ItemRevision] , 
                clientID        =     ["0"]
            }
        loTypeName                             =        "LogicalObject2",
        loQueryNameValues                  =           [<"Included logical object ID2", "ccUid1">, 
                                                                     <"Included logical object ID3", "ccUid2">]
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs list of size = 1
        and element = 1 contains 1 instance(s) of "LogicalObject2". 
        
        One instance of the included logical object: "LogicalObjectDomainB" will be returned if the root ItemRevision
        object of the included logical object satisfies the configuration context specified by "ccUid1".
        
        One instance of the included logical object: "LogicalObjectDomainC" will be returned if the root ItemRevision
        object of the included logical object satisfies the configuration context specified by "ccUid2".
        
        The presented properties on the instances of "LogicalObject2" and its included logical objects:
        "LogicalObjectDomainB" and "LogicalObjectDomainC" will be returned through Service Data. 
        
        Error 39040 - "TYPE_lo_instance_not_found" is stored in the partial error list of service data with "Logical
        Object type name +  root object client id" as the key which helps the caller identify the logical object type
        and root combination for which the logical object instance is not found.
        
        Use Case 4:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {    
                rootObject         =     [Item], 
                clientID            =     ["0"]
            }
        
            RootObject[1]= {    
                  rootObject             =     [Item], 
                clientID                =     ["1"]
            }
        loTypeName                      =      "LogicalObject1"   
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs listof size = 1
        and element = 1 contains 2 instance(s) of "LogicalObject1" and "LogicalObject1". 
        
        The instance(s) of the included logical object: "LogicalObjectDomainA" per main logical object instance will be
        returned for all the  root ItemRevision object(s) which are reached through Included Logical Object Navigation
        Path.
        
        The presented properties on the instances of  "LogicalObject1" and its included logical object:
        "LogicalObjectDomainA" will be returned through Service Data.
        
        Use Case 5:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {
                 rootObject         =     [Item], 
                clientID                =     ["0"]
            }
        
            RootObject[1]= { 
                rootObject         =     [ItemRevision], 
                clientID                =     ["1"]
            }
            loTypeName                     =      "LogicalObject1"
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs listof size = 1
        and element = 1 contains 1 instance(s) of "LogicalObject1". 
        
        The instance(s) of the included logical object: "LogicalObjectDomainA" for the main logical object instance
        will be returned for all the  root ItemRevision object(s) which are reached through Included Logical Object
        Navigation Path.
        
        The presented properties on the instances of  "LogicalObject1" and its included logical object:
        "LogicalObjectDomainA" will be returned through Service Data. 
        
        Error 39040 - is stored in the partial error list of service data with "Logical Object type name +  root object
        client id" as the key which helps the caller identify the logical object type and root combination for which
        the logical object instance is not found.
        
        Use Case 6:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {
                rootObject             =     [Item] ,  
                clientID                =     ["0"],
            }
        loTypeName                           =             "Logical Object 1" 
        loQueryNameValues                  =           [<"Included logical object ID1", "ccUid1">]
        }
        
        GetLogicalObjectInput3[1] = { 
            RootObject[1]= {
                rootObject             =     [ItemRevision],
                clientID                =     ["0"],
            }
        loTypeName                         =         "LogicalObject2",
        loQueryNameValues             =           [ <"Included logical object ID2", "ccUid2">]   
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs listof size = 2
        and element = 1 returns 1 instance(s) of "LogicalObject1". 
        
        One instance of the included logical object: "LogicalObjectDomainA" will be returned if the root ItemRevision
        object of the included logical object satisfies the configuration context specified by "ccUid1".
        
        The presented properties on the instances of "LogicalObject1" and its included logical object:
        "LogicalObjectDomainA" will be returned through Service Data. 
        
        and element = 2 returns 1 instance(s) of "LogicalObject2". 
        
        One instance of the included logical object: "LogicalObjectDomainB" will be returned if the root ItemRevision
        object of the included logical object satisfies the configuration context specified by "ccUid2".
        
        The instance(s) of the included logical object: "LogicalObjectDomainC" for the main logical object instance
        will be returned for all the  root ItemRevision object(s) which are reached through Included Logical Object
        Navigation Path.
        
        The presented properties on the instances of "LogicalObject2" and its included logical objects:
        "LogicalObjectDomainB" and "LogicalObjectDomainC" will be returned through Service Data. 
        
        Use Case 7:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {
                rootObject         =    [Item] , 
                clientID        =     ["0"]
            }
        loTypeName                      =      "Logical Object 1"
        }
        
        GetLogicalObjectInput3[1] = { 
            RootObject[0]= {
                rootObject         =     [Item] , 
            clientID                    =     ["0"]
            }
        loTypeName                      =      "LogicalObject2"   
        }
        
        GetLogicalObjectInput3[2] = { 
            RootObject[0]= {
                rootObject         =     [Item] , 
                clientID                =     ["0"]
            }
        loTypeName                      =      "LogicalObject3"}
        
        then GetLogicalObjectResponse3 structure would contain loOutputs listof size = 3
        and element = 1 returns 1 instance(s) of "LogicalObject1". 
        
        The instance(s) of the included logical object: "LogicalObjectDomainA" for the main logical object instance
        will be returned for all the  root ItemRevision object(s) which are reached through Included Logical Object
        Navigation Path.
        
        The presented properties on the instances of "LogicalObject1" and its included logical object:
        "LogicalObjectDomainA" will be returned through Service Data. 
        
        and element = 2 contains 0 instance(s)
        
        and element = 3 contains 1 instance(s) of "LogicalObject3". 
        
        The presented properties on  the instance of  "LogicalObject3" will be returned through Service Data.
        
        Error 39040 - "TYPE_lo_instance_not_found" is stored in the partial error list of service data with "Logical
        Object type name +  root object client id" as the key which helps the caller identify the logical object type
        and root combination for which the logical object instance is not found.
        
        Use Case 8:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {
                rootObject         =     [ItemRevision] , 
                clientID                =     ["0"]
                }
        loTypeName                       =      "Logical Object 4"   
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs listof size = 1
        and element = 1 returns 1 instance(s) of "Logical Object 4".
        
        The instance(s) of the included logical object: "LogicalObjectDomainD" for the main logical object instance
        will be returned for all the  root ItemRevision object(s) which are reached through Included Logical Object
        Navigation Path.
        The ICO data for the root ItemRevision object of "LogicalObject4" and the ICO data for the root ItemRevision
        object of the included logical object: "LogicalObjectDomainD" will be also returned.
        The presented properties on the instances of  "LogicalObject4" and its included logical object:
        "LogicalObjectDomainD"  will be returned through Service Data. 
        
        Use Case 9:
        
        GetLogicalObjectInput3[0] = { 
            RootObject[0]= {
                rootObject         =     [ItemRevision] , 
                clientID                =     ["0"],
            }
        loTypeName                         =             "Logical Object 4",
        loQueryNameValues                  =           [<"Included logical object ID4", "ccUid1">]
        }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs listof size = 1
        and element = 1 returns 1 instance(s) of "Logical Object 4".
        
        One instance of the included logical object: "LogicalObjectDomainD" will be returned if the root ItemRevision
        object of the included logical object satisfies the configuration context specified by "ccUid1".
        The ICO data for the root ItemRevision object of "LogicalObject4" and the ICO data for the root ItemRevision
        object of the included logical object: "LogicalObjectDomainD" will be also returned.
        The presented properties on the instances of "LogicalObject4" and its included logical object:
        "LogicalObjectDomainD" will be returned through Service Data.
        
        Use Case 10:
        
        GetLogicalObjectInput3[0] = { 
        RootObject[0]= {    
        rootObject         =    [Item],
        clientID        =     ["0"]
        }
        loTypeName         =      "LogicalObject9",   
        loQueryNameValues      =           [<"Member ID1", "ccUid1">]
         }
        
        then GetLogicalObjectResponse3 structure would contain loOutputs list of size = 1
        and element = 1 contains 1 instance(s) of "LogicalObject9". 
        One instance of the logical object: "LogicalObject9" will be returned if the Member Revision satisfies the
        configuration context specified by "ccUid1".
        The presented properties on the instances of "LogicalObject9" will be returned through Service Data.
        """
        return cls.execute_soa_method(
            method_name='getLogicalObjectsWithContext',
            library='Core',
            service_date='2018_11',
            service_name='LogicalObject',
            params={'loInputs': loInputs},
            response_cls=GetLogicalObjectResponse3,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Cad._2013_05.DataManagement import FeatureQueryInfo, QueryPartRelatedFeaturesResponse
from typing import List
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def queryPartRelatedFeatures(cls, featureQueryInfo: List[FeatureQueryInfo]) -> QueryPartRelatedFeaturesResponse:
        """
        Return the PartToPart Relation information for a set of ItemRevisions.
        
        Use cases:
        User is using the NX Relation Browser. They select some items and want to know what the part to part relations
        are.
        """
        return cls.execute_soa_method(
            method_name='queryPartRelatedFeatures',
            library='Internal-Cad',
            service_date='2013_05',
            service_name='DataManagement',
            params={'featureQueryInfo': featureQueryInfo},
            response_cls=QueryPartRelatedFeaturesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Query._2019_06.SavedQuery import BusinessObjectQueryInput3
from typing import List
from tcsoa.gen.Query._2007_09.SavedQuery import SavedQueriesResponse
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def executeBOQueriesWithSort(cls, inputs: List[BusinessObjectQueryInput3]) -> SavedQueriesResponse:
        """
        This operation executes business object queries with sorting options.
        """
        return cls.execute_soa_method(
            method_name='executeBOQueriesWithSort',
            library='Query',
            service_date='2019_06',
            service_name='SavedQuery',
            params={'inputs': inputs},
            response_cls=SavedQueriesResponse,
        )



from __future__ import annotations

from tcsoa.base import TcService
from tcsoa.gen.Internal.Structure._2020_12.WhereUsed import WhereUsedInput, WhereUsedOutput


class WhereUsedService(TcService):

    @classmethod
    def getWhereUsedInfo(cls, whereUsedInput: WhereUsedInput) -> WhereUsedOutput:
        """
        This operation returns the parent ItemRevision object under which input ItemRevision is created as a child
        component. The parent objects are configured based on input RevisionRule. It also returns column configuration
        information for input ClientScopeURI.
        
        Use Case:
        User wants to perform impact analysis by selecting ItemRevision to know under which parent assemblies
        ItemRevision is created as child component.
        """
        return cls.execute_soa_method(
            method_name='getWhereUsedInfo',
            library='Internal-Structure',
            service_date='2020_12',
            service_name='WhereUsed',
            params={'whereUsedInput': whereUsedInput},
            response_cls=WhereUsedOutput,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, EPMTask
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class RGBValue(TcBaseObj):
    """
    Structure for representing color using RGB values, used in structure FilterData.
    
    :var redValue: The Red component of color. Legal range is 0-255.
    :var greenValue: The Green component of color. Legal range is 0-255.
    :var blueValue: The Blue component of color. Legal range is 0-255.
    """
    redValue: float = 0.0
    greenValue: float = 0.0
    blueValue: float = 0.0


@dataclass
class TypeData(TcBaseObj):
    """
    Structure containing the type information.
    
    :var typeName: The name of the type.
    :var displayName: The display name of the type.
    """
    typeName: str = ''
    displayName: str = ''


@dataclass
class WorkflowGraphInput(TcBaseObj):
    """
    Structure represents the parameters required to get the workflow.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    :var selection: The object for which the latest associated workflow to be retrieved.
    :var workflowGraphInfo: A map (string, list of strings) of workflow related information like zoom level etc.Valid
    keys are zoomLevel and layout. The valid value for zoomLevel is any percentage. Valid values for layout are
    Incremental, Orthographic,Tree,Left To Right, Right To Left.
    """
    clientId: str = ''
    selection: BusinessObject = None
    workflowGraphInfo: NameValuePair = None


@dataclass
class WorkflowGraphLegendData(TcBaseObj):
    """
    Structure represents the output data of getWorkflowGraphLegend operation.
    
    :var name: The name of the Legend.
    :var displayName: The localized display name of the Legend.
    :var defaultLayout: The default layout of the view. The values are  Incremental, Orthographic,Tree,Left To Right
    andRight To Left   This value is utilized by the Graph component.
    :var objectGroup: A list of filters associated with objects.
    :var relationGroup: A list of filters associated with relations.
    """
    name: str = ''
    displayName: str = ''
    defaultLayout: str = ''
    objectGroup: List[FilterData] = ()
    relationGroup: List[FilterData] = ()


@dataclass
class WorkflowGraphLegendResponse(TcBaseObj):
    """
    Structure represents the output parameters of getWorkflowGraphLegend operation.
    
    :var legendData: The list of  WorkflowGraphLegendData structures.The legend data will be created based on the XML
    configuration file. This XML file is read from the dataset specified in the preference
    WRKFLW_graph_legend_configuration_file_name.
     
    
     
     
    
    :var presentationRulesXML: The information about how a Node or an Edge is to be rendered based on the type it
    represents is defined in this XML. For instance, a Node representing the EPMTask type is shown as a Tile.  This XML
    is read from the dataset specified  in the preference WRKFLW_graph_presentation_rules_file_name.
    :var presentationStylesXML: The information about shapes, size, line style, arrowstyle etc.    This will be used
    for representation of nodes and edges. This XML is read from the dataset specified  in the preference
    WRKFLW_graph_presentation_styles_file_name.
    :var serviceData: The Service Data.
    """
    legendData: List[WorkflowGraphLegendData] = ()
    presentationRulesXML: str = ''
    presentationStylesXML: str = ''
    serviceData: ServiceData = None


@dataclass
class WorkflowGraphOutput(TcBaseObj):
    """
    Structure represents the output of getWorkflowGraphLegend operation.
    
    :var clientId: The clientId from the input  WorkflowGraphInput structure. This value is unchanged from the input,
    and can be used to identify this response element with the corresponding input element.
    :var workflowGraphInfo: A map (string, list of strings) of diagram related information.
    Valid keys are zoomLevel and layout. The valid value for zoomLevel is any percentage. Valid values for layout are
    Incremental, Orthographic,Tree,Left To Right, Right To Left.
    :var elementData: A list of elements to be shown in the workflow.
    :var edgeData: A list of relations to be shown in the workflow graph. A relation represents the information about
    the connection between the elements in workflow graph.
    """
    clientId: str = ''
    workflowGraphInfo: NameValuePair = None
    elementData: List[ElementData] = ()
    edgeData: List[EdgeData] = ()


@dataclass
class WorkflowGraphResponse(TcBaseObj):
    """
    Structure represents the output parameters of getWorkflowGraph operation.
    
    :var output: A list of WorkflowGraphOutput structures.
    :var serviceData: The Service Data.
    """
    output: List[WorkflowGraphOutput] = ()
    serviceData: ServiceData = None


@dataclass
class EdgeData(TcBaseObj):
    """
    Structure represents the relation to be shown on the graph.
    
    :var end1Element: The primary end of the relation.
    :var end2Element: The secondary end of the relation.
    :var edge: The relation object.
    :var edgeInfo: The information (string, list of strings)  about the relation. Valid key is edgeType and the valid
    value is the type of the relation. This information is used by graphical framework for rendering the Edge. An
    example is:  An edge type defined with name "FailSuccessor" can be configured to render a line with red color with
    dash style to show the fail path in workflow graph.
    """
    end1Element: EPMTask = None
    end2Element: EPMTask = None
    edge: BusinessObject = None
    edgeInfo: NameValuePair = None


@dataclass
class ElementData(TcBaseObj):
    """
    Structure represents the element to be shown on the graph.
    
    :var element: The EPMTask object to be shown in the workflow.
    :var elementInfo: The information (string, list of strings) about the element. Valid key is elementType and the
    valid value is the type of the element.  This information is used by graphical framework for rendering the Element.
    An example is an element type defined as "EPM_started" can be configured to be render the element with yellow color
    frame in workflow graph.
    """
    element: WorkspaceObject = None
    elementInfo: NameValuePair = None


@dataclass
class FilterData(TcBaseObj):
    """
    Structure for representing information of a filter, including filter name, RGB values to color the legend of the
    filter at client side.
    
    :var name: The name of the filter.
    :var displayName: The display name of the filter.
    :var types: The object or relation type information to be shown in the legend.
    :var color: The color used for legend of this filter on the client.
    """
    name: str = ''
    displayName: str = ''
    types: List[TypeData] = ()
    color: RGBValue = None


"""
NameValuePair is a map of property name (key) and property values (values) in string format.
"""
NameValuePair = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.Internal.StructureManagement._2014_12.BrokenLinks import BrokenLinkReplacementResponse, BrokenLinkSearchInput
from tcsoa.gen.Internal.StructureManagement._2014_12.StructureVerification import GetPropPropagationStatusDetailsResp, ImpactedLinesCriteria, GetStructureChangeDetailsResponse, StructureChangeDetailsElement, CreateOrUpdatePropagationDetailsResp, CreateOrUpdateReviewStatusResponse, FindReviewStatusResponse, GetPropPropagationStatusDetailsIn, GetStructureChangeImpactedLinesResponse, CreateOrUpdatePropagationDetailsIn, CreateOrUpdateReviewStatusIn, FindReviewStatusIn
from typing import List
from tcsoa.gen.Internal.StructureManagement._2014_12.Restructure import ReplaceItemsParameter
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class StructureVerificationService(TcService):

    @classmethod
    def getPropertyPropagationStatusDetails(cls, input: List[GetPropPropagationStatusDetailsIn]) -> GetPropPropagationStatusDetailsResp:
        """
        This method is used where there are a set of lines that have property changes associated with them and a set of
        target lines where these property changes might be propagated to.  The method takes in these two sets of lines
        and the property changes.  The method will return the current values of the properties for all the lines and
        the current propagation status for each of the property changes.
        In the context of Change Tracker, this operation will be called when a user selects an impacted line in the
        impacted panel showing property changes.  Change Tracker is a  feature in Manufacturing Process Planner for
        finding and managing changes to structures.
        
        """
        return cls.execute_soa_method(
            method_name='getPropertyPropagationStatusDetails',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=GetPropPropagationStatusDetailsResp,
        )

    @classmethod
    def getStructureChangeDetails(cls, detailsElements: List[StructureChangeDetailsElement]) -> GetStructureChangeDetailsResponse:
        """
        This operation gets detailed information about a changed value on the input line.  The types of changes tracked
        are changes performed under incremental change, revision effecitivity, occurrence effectivity or a given time
        period.  Currently the method only supports changes performed under incremental change.  The input argument is
        the line that changed, the value that changed and configuration information on how to find that line in a
        different configuration.  The operation returns the changed value of that line in the other specified
        configuration.
        """
        return cls.execute_soa_method(
            method_name='getStructureChangeDetails',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='StructureVerification',
            params={'detailsElements': detailsElements},
            response_cls=GetStructureChangeDetailsResponse,
        )

    @classmethod
    def getStructureChangeImpactedLines(cls, impactedLinesCriteria: List[ImpactedLinesCriteria]) -> GetStructureChangeImpactedLinesResponse:
        """
        This operation finds impacted lines in the target structure that are affected by changes made in the source
        structure.   The type of changes made in the source structure that are supported are the following:
        
        - Incremental changes
        - Revision effectivity changes
        - Occurrence effectivity changes
        - Time period changes
        
        
        
        The operation returns a list of impacted lines and affected values.
        """
        return cls.execute_soa_method(
            method_name='getStructureChangeImpactedLines',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='StructureVerification',
            params={'impactedLinesCriteria': impactedLinesCriteria},
            response_cls=GetStructureChangeImpactedLinesResponse,
        )

    @classmethod
    def createOrUpdatePropagationDetails(cls, input: List[CreateOrUpdatePropagationDetailsIn]) -> CreateOrUpdatePropagationDetailsResp:
        """
        This method is needed in situations where there is a source structure and a target structure and some
        equivalent lines between the two structures.  This operation sets or updates the propagation status for a given
        set of target lines.  The actual target lines are determined by the scope and search parameters passed as
        inputs to the method. In the context of Change Tracker, it is called every time when the user changes the
        Review Status value of an impacted line.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdatePropagationDetails',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=CreateOrUpdatePropagationDetailsResp,
        )

    @classmethod
    def createOrUpdateReviewStatus(cls, input: List[CreateOrUpdateReviewStatusIn]) -> CreateOrUpdateReviewStatusResponse:
        """
        This operation sets summary review status for the changes happened on the BOMLine objects in the source
        structure. This summary review status indicate if the selected scopes in the impacted target structure have
        been processed and how they are processed.
        
        Use cases:
        This operation is needed in situations where there is a source structure and impacted target structure and the
        user needs to set the summary review status associated with specified target scopes. In the context of the
        Change Tracker tool in Manufacturing Process Planner,  it is called every time impacted target scope is set or
        changed, to display the summary review status of each change.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateReviewStatus',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=CreateOrUpdateReviewStatusResponse,
        )

    @classmethod
    def findReviewStatus(cls, input: List[FindReviewStatusIn]) -> FindReviewStatusResponse:
        """
        This operation takes as input the BOMLine objects in a source structure having changes, and returns summary
        review status objects for currently selected target structure scopes that these changes may have impact on.
        
        Use cases:
        This operation is needed in situations where there is a source structure and impacted target structure, and the
        user needs to find the summary review status associated with specified target scopes. In the context of the
        Change Tracker tool in Manufacturing Process Planner,  it is called every time the impacted target scope is set
        or changed, to display the summary review status of each change.
        """
        return cls.execute_soa_method(
            method_name='findReviewStatus',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=FindReviewStatusResponse,
        )


class RestructureService(TcService):

    @classmethod
    def replaceItems(cls, replaceInputs: List[ReplaceItemsParameter]) -> ServiceData:
        """
        Replaces an ItemRevision representing a node in the structure with replacement ItemRevision based on the given
        option. The options are
        - Only the selected    Replaces a single selected BOMLine.
        - All sibling occurrences of the selected in the parent assembly. Replaces all BOMLines in the immediate parent
        assembly, referencing the same ItemRevision.
        - All occurrences of the selected in the whole structure. Replaces all BOMLines in the entire structure,
        referencing the same ItemRevision.
        
        
        
        The following restrictions apply to this operation:
        - The preference PS_replace_with_substructure is applied
        - If the selected line has substitutes, the operation will fail.
        - If the selected line or its parent is linked to variant Item the operation will fail.
        - If it is a variant ItemRevision, the ItemRevision to replace it should be a matching variant ItemRevision.
        
        
        
        Note:
        The operation only supports Item objects. Objects of GeneralDesignElement are not supported.
        
        Use cases:
        1. Replace all occurrences in the structure:
        A user wants to replace all occurrences of a ItemRevision in a structure. The user calls the operation with the
        selected BOMLine and replacement ItemRevision. All the occurrences for the ItemRevision in the structure will
        be replaced with the replacement ItemRevision.
        
        2. Replace all occurrences in the structure with partial errors:
        A user wants to replace all occurrences of an ItemRevision in a structure. BOMView Revision of some of the
        occurrences are released. The user calls the operation with selected BOMLine and replacement ItemRevision. The
        occurrences of an ItemRevision whose BOMView Revision is released will fail to be replaced but the other
        ItemRevisions will be replaced. All the failures will be reported to the user for any further action.
        
        3. Replace sibling occurrences:
        A user wants to replace all sibling occurrences of a ItemRevision in the immediate parent assembly. The user
        calls the operation with the ItemRevision and replacement ItemRevision. All the sibling occurrences of the
        ItemRevision in the immediate parent assembly will be replaced with the replacement ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='replaceItems',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='Restructure',
            params={'replaceInputs': replaceInputs},
            response_cls=ServiceData,
        )


class BrokenLinksService(TcService):

    @classmethod
    def getBrokenLinkAndReplacements(cls, brokenLinkInput: BrokenLinkSearchInput) -> BrokenLinkReplacementResponse:
        """
        This service operation finds the broken links in a Bill Of Process (BOP) and searches for replacement
        candidates in the given product structure. The search is based on the specified criteria. 
        This service optionally repairs the broken links if it finds a single replacement candidate.
        
        
        Use cases:
        User has some part consumption in a BOP and accidently some of the parts are removed or replaced from the
        source product structures. This creates broken link in BOP which can be repaired through this service.
        Use Case 1: Find broken link and replacement candidates 
        Description: User can select the root of BOP and find all broken links. Additionally, replacement candidates
        from the product structure are searched based on the search criteria. The search criteria cantains property
        names, their values and whether the criterion is mandatory or optional. User can select the replacement
        candidate to repair the broken links.
        
        Use case 2 : Find broken link and replacement candidates with auto repair
        Description: User can select the option to repair the broken links automatically. This would happen when only
        one replacement candidate is found for the given search criteria.
        """
        return cls.execute_soa_method(
            method_name='getBrokenLinkAndReplacements',
            library='Internal-StructureManagement',
            service_date='2014_12',
            service_name='BrokenLinks',
            params={'brokenLinkInput': brokenLinkInput},
            response_cls=BrokenLinkReplacementResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Query._2008_06.Finder import FindObjectsInput, FindObjectsResponse
from tcsoa.base import TcService


class FinderService(TcService):

    @classmethod
    def findObjectsByClassAndAttributes(cls, input: FindObjectsInput) -> FindObjectsResponse:
        """
        Returns a list of values or objects for a specific class type with certain attributes and values.
        
        Use cases:
        Search for objects using given class type, attribute names and values.
        """
        return cls.execute_soa_method(
            method_name='findObjectsByClassAndAttributes',
            library='Internal-Query',
            service_date='2008_06',
            service_name='Finder',
            params={'input': input},
            response_cls=FindObjectsResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportDataModelResponse(TcBaseObj):
    """
    Holds the response data of the 'exportDataModel' operation.
    
    :var modelXmlFileTicket: The file ticket for the output xml containing the data model extracted.
    :var logFileTicket: The file ticket for the extractor log file.
    :var serviceData: This contains the status of the operation. Possible error codes are , 
    - 216007: Extraction of data model failed
    
    """
    modelXmlFileTicket: str = ''
    logFileTicket: str = ''
    serviceData: ServiceData = None


@dataclass
class ImportDataModelResponse(TcBaseObj):
    """
    The ImportDataModelResponse structure holds the output data from running the deployDataModel operation (which
    replaced the deprecated importDataModel operation).
    
    :var logFileTickets: The file ticket corresponding to output log/logs from the deployDataModel operation.
    :var serviceData: This contains the status of the operation.
    """
    logFileTickets: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Requirementsmanagement._2009_10.RequirementsManagement import StdNoteInput, SetStdNoteDetails, SetStdNoteResponse, OpenStdNoteResponse
from tcsoa.base import TcService


class RequirementsManagementService(TcService):

    @classmethod
    def openStdNote(cls, input: List[StdNoteInput]) -> OpenStdNoteResponse:
        """
        This operation helps to open Fnd0ParamReqment object, or its Revision Fnd0ParamReqmentRevision contents in
        Teamcenter MS Word view. User will get the note text associated with the selected Fnd0ParamReqmentRevision
        allowing editing in that view. Opening Fnd0ParamReqment/ Fnd0ParamReqmentRevision happens in two different ways:
        1.    In context with 'Fnd0ListsParamReqments' relation: In this case, operation gives the Parameter/ value
        pairs selected in context for the parent object of 'Fnd0ListsParamReqments', allowing editing the values.
        2.    Without context: In this case, it gives note text associated for the Fnd0ParamReqmentRevision for
        view/edit purpose.
        
        
        Use cases:
        1.    Suppose user created Fnd0ParamReqment object, and now wants to see/edit note text of it, then opening
        Teamcenter MS Word view, user will see it, and can edit it.
        2.    Suppose user has attached any Fnd0ParamReqment/Fnd0ParamReqmentRevision object to any other
        Item/ItemRevision object with 'Fnd0ListsParamReqments' relation, and now wants to edit/view parameter values
        which are set while attaching this Fnd0ParamReqment/Fnd0ParamReqmentRevision, then opening Teamcenter MS Word
        view will show it.
        """
        return cls.execute_soa_method(
            method_name='openStdNote',
            library='Requirementsmanagement',
            service_date='2009_10',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=OpenStdNoteResponse,
        )

    @classmethod
    def setStdNote(cls, input: List[SetStdNoteDetails]) -> SetStdNoteResponse:
        """
        Sets the parameters and their values on Standard Note/Parametric Requirement.  This SOA operation can set
        values on one or more Standard Note/Parametric Requirement in one operation call. When any Standard
        Note/Parametric Requirement attached to any ItemRevision it will get attached with relation
        Fnd0ListsParamRequirements (Parametric Requirements Lists). In that context if that Standard Note/Parametric
        Requirement object is selected, and edited in MS Word view, then saving of editing values from this view will
        be set on this Standard Note/Parametric Requirement using this SOA. This SOA will set those parameters and
        their values on given relation object.
        
        Use cases:
        You can edit, and set Standard Note/Parametric Requirement Parameter values using MS Word view in Teamcenter.
        This view can be launched using Window->Show view->Other->Teamcenter->MS Word
        """
        return cls.execute_soa_method(
            method_name='setStdNote',
            library='Requirementsmanagement',
            service_date='2009_10',
            service_name='RequirementsManagement',
            params={'input': input},
            response_cls=SetStdNoteResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2007_06.Authorization import AuthorizationInfo, AccessorInfo, AccessorAccessibleNamesList
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Internal.Administration._2007_06.PreferenceManagement import PreferencesDeleteInput, PreferenceResponse, PreferencesImportInput, PreferenceExportResponse, PreferencesExportInput
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class PreferenceManagementService(TcService):

    @classmethod
    def importPreferences(cls, importPrefs: PreferencesImportInput) -> ServiceData:
        """
        Imports preferences from the specified input file, at the specified location (as seen from the end-user), for
        the specified categories and following the specified import action.
        """
        return cls.execute_soa_method(
            method_name='importPreferences',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='PreferenceManagement',
            params={'importPrefs': importPrefs},
            response_cls=ServiceData,
        )

    @classmethod
    def createPreferenceCategories(cls, categoryNames: List[str]) -> ServiceData:
        """
        Creates a new preference category for each provided category name. 
        Only Site administrators can create a new category. 
        A category helps to sort preferences by categories. For example, all security related preferences are
        categorized under a category called "Security". 
        Teamcenter provides most of the category out-of-the-box. Customers can add new categories as required. 
        If no category is provided while creating a preference, it is stored under the "General" category.
        
        Use cases:
        Create a new preference category:
        Site administrator can create a new category of type "MyReservation" using the createPreferenceCategories
        operation by providing a unique name (e.g. "MyReservation") as the category name. Multiple names can be
        provided to create more than one category.
        """
        return cls.execute_soa_method(
            method_name='createPreferenceCategories',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='PreferenceManagement',
            params={'categoryNames': categoryNames},
            response_cls=ServiceData,
        )

    @classmethod
    def deletePreferences(cls, deletePrefs: List[PreferencesDeleteInput]) -> ServiceData:
        """
        This operation is deprecated starting Tc10.0.0. Please use the 'deletePreferencesAtLocations' operation
        instead. 
        
        It enables the deletion of preference instances at any location, provided the logged-in user has the needed
        privileges: users can delete their own preference instances; administrators have the additional possibility to
        delete at other locations (site level, group level, role level, and other users) by specifying a target object
        in the 'PreferencesDeleteInput' structure.
        
        Use cases:
        1. Users to delete a preference instance:
        - Either provide a preference name and preference location as "USER" as part of 'PreferenceDeleteInput'
        structure,
        - Or provide a preference name and user object UID as part of 'PreferenceDeleteInput' structure. 
        
        
        
        2. Group or site administrators to delete a preference instance for location other than their user: 
        Provide a preference name and an object UID for the user, role or group from which the preference instance
        needs to be removed. The administrator must have the needed privilege on the targeted location.
        """
        return cls.execute_soa_method(
            method_name='deletePreferences',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='PreferenceManagement',
            params={'deletePrefs': deletePrefs},
            response_cls=ServiceData,
        )

    @classmethod
    def exportPreferences(cls, exportPrefs: PreferencesExportInput) -> PreferenceExportResponse:
        """
        Exports preferences of the logged-in user for the specified location (as seen from the logged-in user) and
        categories.
        """
        return cls.execute_soa_method(
            method_name='exportPreferences',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='PreferenceManagement',
            params={'exportPrefs': exportPrefs},
            response_cls=PreferenceExportResponse,
        )

    @classmethod
    def getModifiedSitePreferences(cls) -> PreferenceResponse:
        """
        Retrieves the modified site preferences. 
        When values of existing out-of-the-box (OOTB) preferences are changed, or when new preference definitions are
        created, they qualify as modified site preferences.
        """
        return cls.execute_soa_method(
            method_name='getModifiedSitePreferences',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='PreferenceManagement',
            params={},
            response_cls=PreferenceResponse,
        )

    @classmethod
    def getNonSessionPreferences(cls, preferenceScope: str, object: BusinessObject) -> PreferenceResponse:
        """
        Get non session preferences given a preference scope and target object. This method can be used by a DBA to ask
        preferences of another user, role or group specifying the scope.
        """
        return cls.execute_soa_method(
            method_name='getNonSessionPreferences',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='PreferenceManagement',
            params={'preferenceScope': preferenceScope, 'object': object},
            response_cls=PreferenceResponse,
        )


class AuthorizationService(TcService):

    @classmethod
    def setAuthorization(cls, inputAuthorization: List[AuthorizationInfo]) -> ServiceData:
        """
        This operation can be used to set authorization rules for the given accessors.  Authorization rules tells what
        administration applications or the administration utilities the given accessor can access in authoring mode.
        The accessor can be either a Group or Role in the Group. Rule domain specifies if the list is either list of
        utilities or list of applications. Valid values for the domain are utility and application.  If some other
        string is specified as rule domain this operation will return error code 290006.  However authorization setting
        for correct domain names will continue. Following are the valid application names and utility names for this
        operation. For more information on authorization rules please refer to Authorization guide in Teamcenter
        documentation.
        
        List of application IDs:
        Organization
        Business_Modeler_IDE
        Access_Manager
        Archive_Restore
        Setup_Wizard
        Workflow_Designer
        PLMXML_Import_Export
        Project
        Subscription_Monitor
        Classification_Admin
        Report_Designer
        Application_Configuration
        EIntegrator_Admin
        Audit_Manager
        Authorization
        Schema_Editor
        Appearance_Configuration
        ADA License
        
        List of utility IDs:
        data_share
        export_recovery
        database_verify
        update_project_data
        data_sync
        dsa_util
        import_export_business_rules
        purge_invalid_subscriptions
        create_change_types
        fsc_admin
        ada_util
        attribute_export
        
        Use cases:
        To set the authorization rules for an accessor through Authorization application.
        """
        return cls.execute_soa_method(
            method_name='setAuthorization',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='Authorization',
            params={'inputAuthorization': inputAuthorization},
            response_cls=ServiceData,
        )

    @classmethod
    def getAuthorization(cls, inputAccessors: List[AccessorInfo]) -> AccessorAccessibleNamesList:
        """
        This operation can be used to get authorization rules for the given accessors.  Authorization rules tells what
        administration applications or the administration utilities the given accessor can access in authoring mode.
        The accessor can be either a Group or Role in the Group. Rule domain specifies if the list is either list of
        utilities or list of applications. Valid values for the domain are "utility" and "application".  If some other
        string is specified as rule domain this operation will return error code 290006.  Following are the valid
        application names and utility names for this operation. For more information on authorization rules please
        refer to Authorization guide in Teamcenter documentation.
        
        Use cases:
        To display authorization rules in authorization application.
        """
        return cls.execute_soa_method(
            method_name='getAuthorization',
            library='Internal-Administration',
            service_date='2007_06',
            service_name='Authorization',
            params={'inputAccessors': inputAccessors},
            response_cls=AccessorAccessibleNamesList,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class PropertyDefinition(TcBaseObj):
    """
    Structure which has all the information needed for a property.
    
    :var name: Name of the Property.
    :var displayName: Display name of the property.
    :var description: Description about the property.
    :var propertyStorageType: Storage type of property. Supported types are
    - Boolean
    - Character
    - Double
    - Integer
    - Date
    - String
    - LongString
    
    
    :var propertyOptions: A map (string, list of strings) which has a list of string options. Key is property option's
    name and value is option values as a list of strings. Supported Options are
    - initialValue - value to populate the attribute. 
    - lowerBound - lower numerical limit for a numerical or alphanumerical attribute. 
    - upperBound - upper numerical limit for a numerical or alphanumerical attribute.
    - maxStringLength - Maximum String length. 0 for non-string datatypes.
    - isArray - Specifies that the attribute is an array of the data of the data type (for example, string). Takes
    boolean value as a string (true/false). Default is false. 
    - unlimited - Indicates that there is no limit on the number of array elements used for the attribute. Default is
    false.
    - maxLength - Specifies the maximum number of array elements allowed for the attribute.
    - isTransient - Does not persist the attribute in the database. Takes boolean value as a string (true/false).
    Default is false.
    - nullsAllowed - Allows an empty value for the attribute. Takes boolean value as a string (true/false). Default is
    true.
    - isUnique - Specifies that the attribute value cannot be duplicated. Takes boolean value as a string (true/false).
    Default is false.
    - followOnExport - For typed and untyped references, exports the referenced object when the attribute is exported.
    Takes boolean value as a string (true/false). Default is false
    - isPublicRead - Grants everyone read access to the attribute. Takes boolean value as a string (true/false).
    Default is false
    - isPublicWrite - Grants everyone write privileges on the attribute. Takes boolean value as a string (true/false).
    Default is false
    - isCandidateKey - Specifies that when exporting an object, send this attribute and rely on the receiving site to
    look up this string in the local database. This is typically used for system administration defined classes such as
    unit of measure where a local administrator may want to control what units exist on his site. Takes boolean value
    as a string (true/false). Default is false
    - exportAsString - Specifies that when exporting an object, export this attribute as a string. Takes boolean value
    as a string (true/false). Default is false
    - noBackpointer - Does not record the attribute in the POM backpointer table. Each time a forward pointer is
    created for an attribute, an inverse record is stored in the POM backpointer table. Therefore, the backpointer
    table keeps a record of where-referenced attributes and is used for where-referenced calls and for a check that
    things are not referenced when an item is deleted. To help system optimization, you may want to use the No
    Backpointer key for those attributes that are unlikely ever to be deleted (such as owning-user, owning-group, or
    dataset-type). Selecting the No Backpointer key saves the POM system from having to check every reference column in
    the table whenever a where-referenced or delete action is called. Takes boolean value as a string (true/false).
    Default is false.
    - isRequired - The Required property allows you to specify whether a value must be entered for a specific object
    property. Takes boolean value as a string (true/false). Default is false.
    
    
    :var propertyType: Specifies type of property. Valid values are:
    - Attribute
    - Runtime
    - Relation
    - Compound
    - Table
    - Name-value
    
    """
    name: str = ''
    displayName: str = ''
    description: str = ''
    propertyStorageType: str = ''
    propertyOptions: StringVectorMap = None
    propertyType: str = ''


@dataclass
class TypeDefinition(TcBaseObj):
    """
    Business object type definition structure used to create the DataModel update Input.
    
    :var name: Name of business object type.
    :var displayName: Display name of business object type.
    :var parentTypeName: Parent type name of the business object type.
    :var typeClassName: Class type name of the business object type.
    :var description: Description of business object type.
    :var createPrimaryBusinessObject: Creates a new class that stores the data for the new business object type.
    :var isAbstract: Disallows the creation of the business object instances in user interfaces.
    :var typeOptions: Map (string, list of strings) containing additional options. Key is BO option's name and value is
    option values as a vector of strings.
    Options supported:
    - component - Component name of this business object type.
    - storeAsLightWeightObject - Store as light weight business object (true/false). Default is false.
    - allowCreateInstanceOfSecBO - Allow instance creation of Secondary Business Object (true/false). Default is true.
    - isExportable - (true/false). Default is false.
    - isUninstantiable - (true/false). Default is false.
    - isUninheritable - (true/false). Default is false.
    
    """
    name: str = ''
    displayName: str = ''
    parentTypeName: str = ''
    typeClassName: str = ''
    description: str = ''
    createPrimaryBusinessObject: bool = False
    isAbstract: bool = False
    typeOptions: StringVectorMap = None


@dataclass
class TypeInput(TcBaseObj):
    """
    Structure which contains all the information to create a new business object type.
    
    :var typeDefinition: Structure which represents the business object type definitions.
    :var typeInputsMap: Map (string, TypeInput) containing all the dependent business object types. Key is business
    object type name and the value is a TypeInput.
    :var dataModelOptions: Map (string, vector<strings>) containing data model options with key as a string and value
    is a vector of strings
    """
    typeDefinition: TypeDefinition = None
    typeInputsMap: TypeInputMap = None
    dataModelOptions: StringVectorMap = None


@dataclass
class AddPropsOnTypeInput(TcBaseObj):
    """
    Input structure to add properties on a business object type.
    
    :var type: Type name on which new properties are being added.
    :var propDefinitions: A list of Property definitions to add properties on a business object type.
    """
    type: str = ''
    propDefinitions: List[PropertyDefinition] = ()


"""
Map with key as a string and value as vector of strings (string, vector<string>).
"""
StringVectorMap = Dict[str, List[str]]


"""
Map which contains TypeInputs. Key is business object type name and value is TypeInput.
"""
TypeInputMap = Dict[str, TypeInput]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ListOfValues
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class InitialLovData(TcBaseObj):
    """
    Initial LOV Data sent to the server during an LOV query.
    
    :var lov: The ListOfValues that is being evaluated;Use this when the lovInput is not possible to construct and need
    to get the values of specified Lov, otherwise pass as null.
    :var lovInput: This is a container of key-value pairs representing the desired entries for different property
    fields for different operations (Create, SaveAs, Revise, etc.). It is used for context-based evaluation of the LOV
    values where the property values are substituted into query criteria. It is also used for interdependent LOV
    evaluation e.g to evaluate interdependent LOV values at dependent levels given the values at the higher levels that
    are populated on the lovInput
    :var propertyName: The name of the Property for which LOV is being evaluated.
    :var filterData: Filter critieria and other search data.
    """
    lov: ListOfValues = None
    lovInput: LOVInput = None
    propertyName: str = ''
    filterData: LovFilterData = None


@dataclass
class LOVBehaviorData(TcBaseObj):
    """
    Container of data such as LOV usage, Style, Lov Column Names. Additionally it  contains interdependent properties
    that can be used for optimization by the client
    
    :var lovUsage: Indicates if LOV is exhaustive (=1) or suggestive(=2) or Range (=3)
    :var style: Possible values are Standard, Range, Hierarchical, Interdependent and Coordinated
    Standard          - A simple list of values
    Interdependent - An interdependent  list of values, where each value has a  nested list of values. A list of States
    would have a list of Cities for each State. For this style of LOV the dependentProps will have a list of property
    names that are associated with each level of the LOV hierarchy
    Hierarchical      - Same as the Interdependent, but only the last level of values is associated with a property
    Coordinated    - This is similar to Interdependent; however levels 2-N will only
    have a single value. When the user selects a value from the first level LOV,
    the system can then fill property values for all levels
    Range - Subset (ranges) of valid values in terms of upper and lower ranges
    :var columnNames: Names of the columns when displayed in clients
    :var descriptionsAttached: This is a list of Boolean values that indicates if the description for each of the
    column properties is attached
    :var dependendProps: Names of interdependent properties. If the LOV is not an interdependent or coordinated LOV,
    this list will be empty
    :var rangeUpperLimit: Upper limit for Range LOV. This is not applicable to any other style of LOV
    :var rangeLowerLimit: Lower limit for Range LOV. This is not applicable to any other style of LOV
    """
    lovUsage: int = 0
    style: str = ''
    columnNames: LOVColumnNames = None
    descriptionsAttached: List[bool] = ()
    dependendProps: List[str] = ()
    rangeUpperLimit: str = ''
    rangeLowerLimit: str = ''


@dataclass
class LOVColumnNames(TcBaseObj):
    """
    Structure containing column names for the LOV value property, LOV description property and the filter attributes
    
    :var lovValueProp: The attribute on the object to be used as the LOV value. When the user selects a row from the
    list of values in the dynamic LOV widget (an item for example), this configuration point tells the system which
    attribute to use as the LOV value. In case of Static LOVs, the name of the property is returned as "lov_value". In
    case of Dynamic LOVs, it is a name as specified in BMIDE.
    :var lovDescrProp: The attribute on the object to be used as the LOV value description. When the user selects a row
    from the list of values in the dynamic LOV widget (an item for example), this configuration point tells the system
    which attribute to use as the LOV value description. In case of Static LOVs, the name of the property is returned
    as "lov_value_description". In case of Dynamic LOVs, it is a name as specified in BMIDE. It can be have empty value
    if it is not specified by BMIDE administrator.
    :var filterProperties: The filter properties would appear as additional columns along with the LOV Value and LOV
    description in the Dynamic LOV
    :var displayNames: Displayable names for each of lovValueProp, lovDescrProp, filterProperties that clients can use
    to display the same
    """
    lovValueProp: str = ''
    lovDescrProp: str = ''
    filterProperties: List[str] = ()
    displayNames: StringMap = None


@dataclass
class LOVData(TcBaseObj):
    """
    This data structure is not for end user consumption. Data in this structure will be used by server for returning
    the next set of LOV values
    
    :var style: Possible values are Standard, Dynamic, Range, Hierarchical, Interdependent and Coordinated
    :var filterData: Filter critieria and other search data.
    :var unProcessedObjects: These are the list of UIDs that will be processed by the server in subsequent calls to get
    next set of LOV values
    :var additionalValuesSkipped: Boolean flag to indicate if additional UIDs (Values in static LOV) are skipped or not
    :var currentIndex: Current index of the LOV results that have been processed.
    :var lovs: LOVs of Dynamic LOV which is being evaluated
    """
    style: str = ''
    filterData: LovFilterData = None
    unProcessedObjects: List[str] = ()
    additionalValuesSkipped: bool = False
    currentIndex: int = 0
    lovs: List[ListOfValues] = ()


@dataclass
class LOVInput(TcBaseObj):
    """
    This represents the operation data.  Operations such as Create, Edit, Revise, Save As, Search or any other
    operation that requires the data to be passed generically to the LOV service operations  have the property values
    represented through the LOVInput for computing a LOV.
    
    :var owningObject: Owning object for the operation in context.
    a. Edit operation - Object being edited.
    b. Save operation - Object being saved.
    c. Revise operation - Object being revised.
    d. SaveAs operation - Object being copied.
    e. Search operation - Saved Query Object for searching.
    f. If an operation does not have an object, specify the value as null and ensure boName is passed for the same. For
    example, during creation, client does not have an object. Therefore specify the business object name of the object
    being created.  For example, if Item object is being created, specify the boName as "Item" and operationName as
    "Create".
    :var boName: Name of the source business object. For example, Item is the source business object for Item
    Descriptors. If the owningObject is not null, then it can be empty. Server can determine the business object name
    from the owningObject. It is mandatory for Create operation where owningObject is null
    :var operationName: Name of the operation  being performed. Valid names are Create, Revise, SaveAs, Edit, Search,
    Save
    :var propertyValues: A map of property names and values (string, vector<string>). The client is responsible for
    converting the values of the different property types (int, float, date .etc) to a string using the appropriate
    toXXXString functions in the SOA client  framework's Property class. Single valued properties will have a single
    value in the value vector, while Multi-valued properties will have multiple values in the value vector.
    """
    owningObject: BusinessObject = None
    boName: str = ''
    operationName: str = ''
    propertyValues: PropertyValues = None


@dataclass
class LOVSearchResults(TcBaseObj):
    """
    This structure contains the LOV results from the getInitialLOVValues or getNextLOVValues operations
    
    :var lovValues: This is a list of LOVValueRow objects. Each LOVValueRow object represents a single row of the LOV
    results. It includes the LOV value property, the description property and all the filter properties (the filter
    properties are the ones whose values are used for filtering the search results based on user input)
    :var moreValuesExist: true indicates there are more values available which will can be retrieved by a call to the
    getNextValues operation
    :var behaviorData: LOV data used to define the UI component behavior. This includes data such as LOV usage, Style,
    Lov Column Names. Additionally it  contains interdependent properties that can be used for optimization by the
    client
    :var lovData: If moreValuesExist is true, this object is passed as input to the getNextValues operation to get the
    next list of LOV search results
    :var serviceData: The ServiceData
    """
    lovValues: List[LOVValueRow] = ()
    moreValuesExist: bool = False
    behaviorData: LOVBehaviorData = None
    lovData: LOVData = None
    serviceData: ServiceData = None


@dataclass
class LOVValueRow(TcBaseObj):
    """
    This represents a row of LOV values. It includes the internal and display values for the various columns for each
    LOV value
    
    :var uid: UID of the object. This applies to dynamic LOVs.  Server sends valid UID for dynamic LOV and empty in
    other LOV cases.. Client can send the same UID when a value is selected to validateLOVValueSelection operation for
    effective validation. This is empty for for non dynamic LOVs
    :var propInternalValues: The internal values of all the properties on a single row of the LOV search results. The
    parseXXX functions in the SOA framework class can be used to retrieve the values for the specific property types
    :var propDisplayValues: The display values of all the properties on a single row of the LOV search results
    :var childRows: Next level of row values in case of hierarchical LOV. This is recursive and can go down multiple
    levels in the hierarchy
    """
    uid: str = ''
    propInternalValues: PropertyValues = None
    propDisplayValues: PropertyValues = None
    childRows: List[LOVValueRow] = ()


@dataclass
class LovFilterData(TcBaseObj):
    """
    Filter Data used in InitialLOVData.
    
    :var filterString: The desired string used to filter the search results. For example, if "Green" is entered as a
    filter string, the results returned for LOV values will include the LOV values that match the query criteria AND
    contain the string "Green" in the LOV Description or any of the filter attributes. Case sensitive nature is
    honoured based on TC_ignore_case_on_search preference.
    :var maxResults: Maximum number of LOV results that server should retrieve from the database
    :var numberToReturn: Number of objects to be processed in case of dynamic LOVs to return LOV values and in other
    LOV cases it is to return the number of LOV values from the LOV results. This number must be less than or equal to
    maxResults
    :var sortPropertyName: Property name on which to sort the results. This is optional
    :var order: 1=sort in ascending order, 2=sort in descending order. Ignored if sortPropertyName is null.
    """
    filterString: str = ''
    maxResults: int = 0
    numberToReturn: int = 0
    sortPropertyName: str = ''
    order: int = 0


@dataclass
class ValidateLOVValueSelectionsResponse(TcBaseObj):
    """
    Response structure indicating validity of LOV value selection and containing updated property values and
    interdependent LOV values
    
    :var propHasValidValues: Indicates whether input property has valid values
    :var dependentPropNames: Names of dependent properties server modified to be updated by client as a results of
    dependency with input property selection. This can be empty.
    :var updatedPropValues: Updated property values. It contains both internal and display values of the updated
    properties.  It can be empty.
    :var serviceData: The service data.
    """
    propHasValidValues: bool = False
    dependentPropNames: List[str] = ()
    updatedPropValues: LOVValueRow = None
    serviceData: ServiceData = None


"""
Map (string, vector<string>) that is a generic container that represents property values. The key is the property name and the value is the string representation of the property value.
"""
PropertyValues = Dict[str, List[str]]


"""
Map of Strings
"""
StringMap = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Manufacturing._2013_12.ResourceManagement import GetStepP21FileCountsResponse, ImportStepP21FilesResponse
from tcsoa.gen.Internal.Manufacturing._2014_12.ResourceManagement import GetVendorCatalogInfo2Response
from tcsoa.gen.Internal.Manufacturing._2014_12.Model import LAResolveAsyncData, LABatchDetails, UILocationInfo, UILocationsInfoResponse, ScopeFlowInfo
from typing import List
from tcsoa.gen.Internal.Manufacturing._2014_12.IPAManagement import SearchDynamicIPAsResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class ResourceManagementService(TcService):

    @classmethod
    def getStepP21FileCounts2(cls, classIds: List[str], catalogRootDirectories: List[str]) -> GetStepP21FileCountsResponse:
        """
        For each identified class, this operation counts all STEP P21 tool component definition files that are stored
        below the specified Generic Tool Catalog vendor catalog root directory and belong to the identified class or
        any of its subclasses.
        """
        return cls.execute_soa_method(
            method_name='getStepP21FileCounts2',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='ResourceManagement',
            params={'classIds': classIds, 'catalogRootDirectories': catalogRootDirectories},
            response_cls=GetStepP21FileCountsResponse,
        )

    @classmethod
    def getVendorCatalogInfo2(cls, catalogTypes: int) -> GetVendorCatalogInfo2Response:
        """
        The multi-value preference "MRMGTCVendorCatalogRootDir" specifies one or more root directories where vendor
        tool catalogs may be stored on the Teamcenter server machine.
        This operation retrieves additional information about valid vendor catalogs contained in these root
        directories. It scans the given root directories for vendor catalogs of the requested type and returns detailed
        information for each valid catalog.
        """
        return cls.execute_soa_method(
            method_name='getVendorCatalogInfo2',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='ResourceManagement',
            params={'catalogTypes': catalogTypes},
            response_cls=GetVendorCatalogInfo2Response,
        )

    @classmethod
    def importStepP21Files2(cls, classId: str, catalogRootDirectory: str, importOptions: int) -> ImportStepP21FilesResponse:
        """
        This operation imports STEP P21 files containing tool component data pertaining to vendor tool catalogs into
        the Classification classes that represent those vendor tool catalogs inside the database. It creates Internal
        Classification objects (ICOs) in those classes, and associated data that represents the tool components.
        Depending on the contents of the vendor catalog directory, it also creates associated items, item revisions,
        datasets, and associated files to store drawings and images that further describe those tool components.
        """
        return cls.execute_soa_method(
            method_name='importStepP21Files2',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='ResourceManagement',
            params={'classId': classId, 'catalogRootDirectory': catalogRootDirectory, 'importOptions': importOptions},
            response_cls=ImportStepP21FilesResponse,
        )


class ModelService(TcService):

    @classmethod
    def getUILocations(cls, contextLines: List[BusinessObject]) -> UILocationsInfoResponse:
        """
        This operation reads the absolute occurrence (Mfg0AbsOcc) and user interface (UI) location (Mfg0UILocation)
        property of Mfg0UILocationForm form attached to the input BOPLine objects, gets the BOPLine corresponding to
        the absolute occurrence and returns the BOPLine along with the UI location. The input BOPLine objects can be of
        type Mfg0BvrProcess, Mfg0BvrOperation, Mfg0BvrWorkarea or Mfg0BvrProcessArea. 
        """
        return cls.execute_soa_method(
            method_name='getUILocations',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='Model',
            params={'contextLines': contextLines},
            response_cls=UILocationsInfoResponse,
        )

    @classmethod
    def laAsyncResolve(cls, laResolveData: LAResolveAsyncData) -> None:
        """
        This service operation resolves logical assignments for a given scope. This is an asynchronous operation.
        Logical assignments are defined on a process (MEProcess) or an operation (MEOP). These contain a user defined
        criteria which can be used to assign parts to these process(s)/operation(s). This operation automatically
        searches the product structure(s) for the parts that satisfy the criteria defined on the logical assignments
        and assigns them to the process/operation.
        """
        return cls.execute_soa_method(
            method_name='laAsyncResolve',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='Model',
            params={'laResolveData': laResolveData},
            response_cls=None,
        )

    @classmethod
    def saveUILocations(cls, uiLocationsInfo: List[UILocationInfo]) -> ServiceData:
        """
        This operation creates Mfg0UILocationForm and attaches it to MEAppearancePathNode of the input
        contextLine(BOPLine) using the Mfg0UILocationFormRel relationship and stores absolute occurrence and user
        interface(UI) location of the BOPLine objects contained in the externalNodeInfo structure. 
        """
        return cls.execute_soa_method(
            method_name='saveUILocations',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='Model',
            params={'uiLocationsInfo': uiLocationsInfo},
            response_cls=ServiceData,
        )

    @classmethod
    def scheduleLAResolve(cls, laResolveData: LAResolveAsyncData, batchDetails: LABatchDetails) -> ServiceData:
        """
        This service operation schedules an asynchronous service operation laAsyncResolve to be executed at a specific
        time. The operation is scheduled to execute once or a number of times depending on the information provided in
        the parameter batchDetails.
        Logical assignments (LAs) are defined on a process (MEProcess) or an operation (MEOP). These contain a user
        defined criteria which can be used to assign parts to these process(s)/operation(s). The service operation
        laAsyncResolve automatically searches the product structure(s) for the parts that satisfy the criteria defined
        on the logical assignments and assigns them to the process/operation.
        """
        return cls.execute_soa_method(
            method_name='scheduleLAResolve',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='Model',
            params={'laResolveData': laResolveData, 'batchDetails': batchDetails},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteScopeFlows(cls, scopeFlowsInfo: List[ScopeFlowInfo]) -> ServiceData:
        """
        This operation removes the Mfg0BvrScopeFlow object and the Mfg0ScopeFlowPredRel relation between the input
        predecessor and the successor BOPLine objects.
        """
        return cls.execute_soa_method(
            method_name='deleteScopeFlows',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='Model',
            params={'scopeFlowsInfo': scopeFlowsInfo},
            response_cls=ServiceData,
        )


class IPAManagementService(TcService):

    @classmethod
    def searchDynamicIPAs(cls, searchScopes: List[BusinessObject], returnOnFirstResult: bool) -> SearchDynamicIPAsResponse:
        """
        This operation searches and returns the dynamic in-process assembly objects present in the hierarchy of the
        search input bop line objects.
        """
        return cls.execute_soa_method(
            method_name='searchDynamicIPAs',
            library='Internal-Manufacturing',
            service_date='2014_12',
            service_name='IPAManagement',
            params={'searchScopes': searchScopes, 'returnOnFirstResult': returnOnFirstResult},
            response_cls=SearchDynamicIPAsResponse,
        )



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_05.StructureVerification import BOMLinePair, AlignmentCheckResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.StructureManagement._2008_05.StructureSearch import SearchScope, StructureSearchResultResponse, SearchExpressionSet
from tcsoa.base import TcService


class StructureSearchService(TcService):

    @classmethod
    def nextSearch(cls, searchCursor: BusinessObject) -> StructureSearchResultResponse:
        """
        This operation gets the next set of search results, which was initialized by the 'startSearch' operation. This
        operation returns the results in batches and needs to be called repeatedly until the flag for search complete
        is true in the response. Input to this operation is the search cursor object which was returned by the
        'startSearch' or a previous call to 'nexSearch' operation.
        
        Use cases:
        A user wants to perform structure search within a particular scope. The user needs to select search
        criteria(s), from the supported list to create search expression and start search operation. Once the initial
        set of a search result is returned, this operation will be used to get the next set of search result.
        
        Exceptions:
        >None
        """
        return cls.execute_soa_method(
            method_name='nextSearch',
            library='StructureManagement',
            service_date='2008_05',
            service_name='StructureSearch',
            params={'searchCursor': searchCursor},
            response_cls=StructureSearchResultResponse,
        )

    @classmethod
    def startSearch(cls, scope: SearchScope, searchExpression: SearchExpressionSet) -> StructureSearchResultResponse:
        """
        Start searching a structure for a given search expression within the scope specified
        """
        return cls.execute_soa_method(
            method_name='startSearch',
            library='StructureManagement',
            service_date='2008_05',
            service_name='StructureSearch',
            params={'scope': scope, 'searchExpression': searchExpression},
            response_cls=StructureSearchResultResponse,
        )

    @classmethod
    def stopSearch(cls, searchCursor: BusinessObject) -> StructureSearchResultResponse:
        """
        This operation stops the current search identified by the search cursor object. The input to the operation is
        the search cursor object which was returned by the 'startSearch' or previous call to 'nextSearch' operation.
        
        Use cases:
        A user wants to perform Cacheless search within a particular scope. The user needs to select  search
        criteria(s) from the supported list to create search expression and start search operation. Once the initial
        batch of search result is returned, this operation can be used to stop the search.
        
        Exceptions:
        >Invalid SearchCursor object is passed in input.
        """
        return cls.execute_soa_method(
            method_name='stopSearch',
            library='StructureManagement',
            service_date='2008_05',
            service_name='StructureSearch',
            params={'searchCursor': searchCursor},
            response_cls=StructureSearchResultResponse,
        )


class StructureVerificationService(TcService):

    @classmethod
    def checkAlignment(cls, input: List[BOMLinePair]) -> AlignmentCheckResponse:
        """
        An alignment connects two occurrences that are to be considered equivalent. They are referred
        to as source and target. An alignment can connect one source
        to multiple targets. Alignment can be used to transfer data from source to target.
        An alignment check is used to determine if the source and target of an alignment have matching data.
        """
        return cls.execute_soa_method(
            method_name='checkAlignment',
            library='StructureManagement',
            service_date='2008_05',
            service_name='StructureVerification',
            params={'input': input},
            response_cls=AlignmentCheckResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SimulationProcessManagementService(TcService):

    @classmethod
    def importSimulationObjects2(cls, processHierarchy: str, workingDirectory: str, itemID: str, objectUID: str, revisionID: str, xmlFileName: str, itemCreationSetting: str, datasetCreationSetting: str) -> ServiceData:
        """
        This internal operation will import the output files generated from execution of simulation tool launch process
        into Teamcenter objects. Based on the input values of parameters itemCreationSetting and datasetCreationSetting
        ItemRevision and Dataset objects will be created or modified. It is pre-requisite that the Simulation
        Administrator or user with DBA privileges must configure the simulation tool and store it in XML format as a
        named reference with the Dataset name specified in the preference CAE_simulation_tool_config_dsname. 
        
        Use cases:
        Use Case 1: Import Simulation Objects with As Needed Item Creation Setting
        When this operation is executed with Item creation setting As Needed, no new Teamcenter Item will be created if
        there exists one as per the process output configuration. If found none, new Item will be created as per the
        process output configuration.
        
        Use Case 2: Import Simulation Objects with Always Item Creation Setting
        When this operation is executed with Item creation setting Always, new Teamcenter Item will be created as per
        the process output configuration.
        
        Use Case 3: Import Simulation Objects with Never Item Creation Setting
        When this operation is executed with Item creation setting Never and if no existing Item is found as per the
        process output configuration, no new Teamcenter Item will be created.
        
        Use Case 4: Import Simulation Objects with As Needed Dataset Creation Setting
        When this operation is executed with Dataset creation setting As Needed, no new Teamcenter Dataset will be
        created if there exists one as per the process output configuration. If found none, new Dataset will be created
        as per the process output configuration.
        
        Use Case 5: Import Simulation Objects with Always Dataset Creation Setting
        When this operation is executed with Dataset creation setting Always, new Teamcenter Dataset will be created as
        per the process output configuration.
        
        Use Case 6: Import Simulation Objects with Never Dataset Creation Setting
        When this operation is executed with Dataset creation setting Never and if no existing Dataset is found as per
        the process output configuration, no new Teamcenter Dataset will be created.
        """
        return cls.execute_soa_method(
            method_name='importSimulationObjects2',
            library='Internal-Cae',
            service_date='2013_12',
            service_name='SimulationProcessManagement',
            params={'processHierarchy': processHierarchy, 'workingDirectory': workingDirectory, 'itemID': itemID, 'objectUID': objectUID, 'revisionID': revisionID, 'xmlFileName': xmlFileName, 'itemCreationSetting': itemCreationSetting, 'datasetCreationSetting': datasetCreationSetting},
            response_cls=ServiceData,
        )


class StructureManagementService(TcService):

    @classmethod
    def propagateCAEModelAttributes(cls) -> bool:
        """
        This operation propagates all selected attributes on a target CAEModel structure, on which the CAE
        Accountability Check was performed. It also attaches the named HTML report Dataset on CAEModel root Item. The
        HTML report contains old value, new value and propagation status of each partially matched attribute. Dataset
        name is driven from CAE_attribute_propagate_summary_dataset_name preference.
        
        
        Use cases:
        This operation is a follow-up operation based on the result from CAE Accountability Check comparison. If the
        user wants to propagate the target CAEModel attributes, then CAE Accountability Check operation should be
        performed with selected attributes between Product and Model structures. Then propagateCAEModelAttributes
        should be called to propagate the partially matched attributes.
        """
        return cls.execute_soa_method(
            method_name='propagateCAEModelAttributes',
            library='Internal-Cae',
            service_date='2013_12',
            service_name='StructureManagement',
            params={},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.Rdv._2010_09.ContextManagement import UpdateSCOResponse, CreateSCOResponse, UpdateSCOInputInfo, CreateSCOInputInfo
from typing import List
from tcsoa.base import TcService


class ContextManagementService(TcService):

    @classmethod
    def updateSCO(cls, inputs: List[UpdateSCOInputInfo]) -> UpdateSCOResponse:
        """
        Updates the Structure Context Object (SCO) based on the inputs attributes. It sets the following properties on
        SCO object which is to be modified: Product Item Revision, Revision Rule, Variant Rule, Work parts, Search
        Criteria Group, Target and Background BOMLine objects from the input structure. This SCO will contain the Item,
        Item revisions, Target BOMLine objects, Background BOMLine objects. The operation is designed to support
        multiple SCOs creations in a single call. (Limitation: Though this operation can update multiple SCO objects
        however it can return reference of only one object). This operation first checks for the local ownership of the
        object to be updated. This operation will fail if null or incorrect reference to existing SCO object is passed
        in the input.
        
        Use cases:
        1. Update an SCO
        You can update an SCO object of type VisStructureContext using 'updateSCO' operation by providing the
        'UpdateSCOInputInfo' structure.
        - Create an SCO, object of StructureContext, using the 'createSCO' operation.
        - Retrieve the reference to StructureContext returned from above step.
        - Modify the required search criteria and populate the 'UpdateSCOInputInfo' structure.
        - Call 'updateSCO' which will modify the existing StructureContext object.
        
        """
        return cls.execute_soa_method(
            method_name='updateSCO',
            library='Rdv',
            service_date='2010_09',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=UpdateSCOResponse,
        )

    @classmethod
    def createSCO(cls, inputs: List[CreateSCOInputInfo]) -> CreateSCOResponse:
        """
        Creates the Structure Context Object (SCO) based on the inputs supplied. It creates an SCO object and then sets
        the following properties on SCO object created: Product Item Revision, Revision Rule, Variant Rule, Work parts
        selected, Search Criteria Group, Target and Background BOMLine objects from the input structure. This SCO will
        contain the Item, Item revisions, Target BOMLine objects, Background BOMLine objects. The operation is designed
        to support multiple SCOs creations in a single call. (Limitation: Though this operation can create multiple SCO
        objects however it can return reference of only one object). The operation will initially create the SCO object
        using the name, type and description. Subsequently it would set the additional parameters supplied through the
        input structure. SCO object would still be created and saved even if setting of the additional parameters is
        not successful.
        
        Use cases:
        1. Create an SCO
        YouYou can create a new SCO object of type VisStructureContext using 'createSCO' operation by providing the
        'CreateSCOInputInfo' structure.
        """
        return cls.execute_soa_method(
            method_name='createSCO',
            library='Rdv',
            service_date='2010_09',
            service_name='ContextManagement',
            params={'inputs': inputs},
            response_cls=CreateSCOResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class DeepCopyData(TcBaseObj):
    """
    Holds the relavent information regarding applicable deep copy rules.
    
    :var otherSideObjectTag: This is a tag representing object on which the deep copy action need to be performed.
    :var relationTypeName: This is a string representing the name the relation that need to be deep copied.
    :var newName: This is a string representing the new name for the new copy of the object represented by
    'otherSideObjectTag'. The value for the 'newName' will be 'NULL' if the 'action' is not 'CopyAsObject' or
    'ReviseAndRelateToLatest'.
    :var action: This is an integer representing the action to be performed on the object represented by
    'otherSideObjectTag'. The values for action are: 'CopyAsObjectType = 0, CopyAsRefType = 1, DontCopyType =2,
    RelateToLatest = 3, ReviseAndRelateToLatest = 4'.
    :var isTargetPrimary: Boolean representing whether the given item revision is a primary object in the relation that
    need to be deep copied.
    :var isRequired: Boolean representing whether the deep information is from a mandatory deep copy rule configured by
    the administrator or not.
    :var copyRelations: Boolean representing whether the Properties on  the Relation if any in the Relation that exists
    between the Primary and the Secondary should be carried forward or not.
    """
    otherSideObjectTag: BusinessObject = None
    relationTypeName: str = ''
    newName: str = ''
    action: int = 0
    isTargetPrimary: bool = False
    isRequired: bool = False
    copyRelations: bool = False


@dataclass
class DeepCopyInfoKey(TcBaseObj):
    """
    Holds the ItemRevision object tag ('itemRevisionTag') and the operation name for which the deep copy information
    should be obtained.
    
    :var itemRevisionTag: The ItemRevision object on which the operation is being performed.
    :var operation: The string representing the operation name (i.e., 'Revise' or 'SaveAs' ).
    """
    itemRevisionTag: ItemRevision = None
    operation: str = ''


@dataclass
class DeepCopyInfoKeyValue(TcBaseObj):
    """
    Store key and value pair for DeepCopyData.
    
    :var key: Structure representing the key for the Deep Copy Info.
    :var deepCopyInfos: The resultant 'deepCopyInfo' values are returned as a list.
    """
    key: DeepCopyInfoKey = None
    deepCopyInfos: List[DeepCopyData] = ()


@dataclass
class DeepCopyInfoResponse(TcBaseObj):
    """
    Holds the response for the 'getDeepCopyInfo' method.
    
    :var values: The resultant 'deepCopyInfo' values are returned as a list.
    :var serviceData: Contains the status of the operation. This operation does not return any error code but
    propagates the error code from lower level functions.
    """
    values: List[DeepCopyInfoKeyValue] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ListOfValues, CadAttrMappingDefinition, ImanRelation, Folder, Dataset, Item, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandFolderForCADPref(TcBaseObj):
    """
    Contains list of RelationAndTypesFilter, number of latest revisions for further filtering and a flag to check
    whether item revision needs to be expanded.
    
    :var expItemRev: Flag to specify if item revisions are to be expanded ( TRUE ) or not ( FALSE ) if found as other
    side objects.
    :var latestNRevs: The number of latest revisions under an Item that should be considered for further filtering
    :var info: A list of RelationAndTypesFilter2
    """
    expItemRev: bool = False
    latestNRevs: int = 0
    info: List[RelationAndTypesFilter2] = ()


@dataclass
class ExpandFoldersForCADItemOutput(TcBaseObj):
    """
    Contains the item expanded and the list of item revisions for the item.
    
    :var outputItem: An item that is in the input folder.
    :var itemRevsOutput: A list of 'ExpandFolderForCADItemRevOutput' which contains information about any item
    revisions that belong to item as specified by 'latestNRevs'.
    """
    outputItem: Item = None
    itemRevsOutput: List[ExpandFoldersForCADItemRevOutput] = ()


@dataclass
class ExpandFoldersForCADItemRevOutput(TcBaseObj):
    """
    Contains the item revision expanded and the results of expanding the item revision.
    
    :var outputItemRevs: An item revision that is in the input folder.
    :var outputDatasets: A list of datasets related to the item revision as specified by the input filter.
    """
    outputItemRevs: ItemRevision = None
    outputDatasets: List[Dataset] = ()


@dataclass
class ExpandFoldersForCADOutput(TcBaseObj):
    """
    Contains the output data for ExpandFoldersForCAD operation.
    
    :var inputFolder: Folder object reference of the folder of interest
    :var fstlvlFolders: A list of Folder object references in the input folder
    :var itemsOutput: A list of ExpandFoldersForCADItemOutput for the item
    :var itemRevsOutput: A list of ExpandFoldersForCADItemRevOutput for the item revision
    """
    inputFolder: Folder = None
    fstlvlFolders: List[Folder] = ()
    itemsOutput: List[ExpandFoldersForCADItemOutput] = ()
    itemRevsOutput: List[ExpandFoldersForCADItemRevOutput] = ()


@dataclass
class ExpandFoldersForCADResponse(TcBaseObj):
    """
    Contains the response for ExpandFoldersForCAD operation.
    
    :var output: A list of ExpandFoldersForCADOutput which has information about the input folder and folders, items
    and itemRevs output found under this folder
    :var serviceData: SOA framework object containing objects that were created, deleted, or updated by the Service,
    plain objects, and service errors/failure information
    """
    output: List[ExpandFoldersForCADOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandGRMRelationsData(TcBaseObj):
    """
    Contains a list of related objects and the relation name used to relate to the input object.
    
    :var otherSideObjects: A list of object references of objects the object of interest is related to.
    :var relationName: The name of the relation used to relate the object of interest to the other otherSideObjects
    """
    otherSideObjects: List[BusinessObject] = ()
    relationName: str = ''


@dataclass
class ExpandGRMRelationsOutput(TcBaseObj):
    """
    Contains the output data for ExpandGRMRelations operation.
    
    :var inputObject: The object reference of the object of interest
    :var otherSideObjData: List of ExpandGRMRelationsData
    """
    inputObject: BusinessObject = None
    otherSideObjData: List[ExpandGRMRelationsData] = ()


@dataclass
class ExpandGRMRelationsPref(TcBaseObj):
    """
    Contains a list of RelationAndTypesFilter and a flag to check whether item revision needs to be expanded.
    
    :var expItemRev: Flag to specify if item revisions are to be expanded ( TRUE ) or not ( FALSE ) if found as other
    side objects.
    :var info: List of RelationAndTypesFilter2
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter2] = ()


@dataclass
class ExpandGRMRelationsResponse(TcBaseObj):
    """
    Top level response structure for ExpandGRMRelations operations.
    
    :var output: A list of ExpandGRMRelationsOutput which has information about the input object and it's filtered
    related object list
    :var serviceData: SOA framework object containing objects that were created, deleted, or updated by the Service,
    plain objects, and service errors/failure information
    """
    output: List[ExpandGRMRelationsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExpandPrimaryObjectsData(TcBaseObj):
    """
    Contains a list of related objects and the relation name or property used to relate to the input object.
    
    :var otherSideObjects: A list of object references of objects the object of interest is related to.
    :var relationName: The name of the relation used to relate the object of interest to the other otherSideObjects
    """
    otherSideObjects: List[BusinessObject] = ()
    relationName: str = ''


@dataclass
class ExpandPrimaryObjectsOutput(TcBaseObj):
    """
    Contains the output data for ExpandPrimaryObjects operation.
    
    :var inputObject: The object reference of the object of interest
    :var otherSideObjData: List of ExpandPrimaryObjectsData
    """
    inputObject: BusinessObject = None
    otherSideObjData: List[ExpandPrimaryObjectsData] = ()


@dataclass
class ExpandPrimaryObjectsPref(TcBaseObj):
    """
    Contains list of RelationAndTypesFilter and a flag to check whether item revision needs to be expanded .
    
    :var expItemRev: Flag to specify if item revisions are to be expanded ( TRUE ) or not ( FALSE ) if found as other
    side objects.
    :var info: List of RelationAndTypesFilter2
    """
    expItemRev: bool = False
    info: List[RelationAndTypesFilter2] = ()


@dataclass
class ExpandPrimaryObjectsResponse(TcBaseObj):
    """
    Contains the response structure for ExpandPrimaryObjects operation.
    
    :var output: A list of ExpandPrimaryObjectsOutput
    :var serviceData: SOA framework object containing objects that were found by the service and service errors/failure
    information
    """
    output: List[ExpandPrimaryObjectsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ExtraObjectInfo(TcBaseObj):
    """
    Form objects that can be created or updated and related to an item, item revision or dataset.
    
    :var object: Object reference for existing object
    :var clientId: Identifier defined by user to track the related object.
    :var relationTypeName: Name of the relation type to the parent object
    :var typeName: Object Type name
    :var attrNameValuePairs: List of AttributeInfos.
    """
    object: BusinessObject = None
    clientId: str = ''
    relationTypeName: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()


@dataclass
class ExtraObjectOutput(TcBaseObj):
    """
    Form objects that can be created or updated and related to an item, item revision or dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var object: Object reference of the created/updated object
    """
    clientId: str = ''
    object: BusinessObject = None


@dataclass
class GenerateAlternateIdsProperties(TcBaseObj):
    """
    Holds the input structure of 'GenerateAlternateIdsProperties'
    
    :var idContext: Object reference of the id context used to generate the alternate id of that context
    :var pattern: Pattern type used to generate the alternate id of that pattern
    :var altIdType: Type of the alternate id to generate
    :var parentAltId: Object reference of the parent alternate id
    :var count: Number of the alternate id to be generated.
    """
    idContext: BusinessObject = None
    pattern: str = ''
    altIdType: str = ''
    parentAltId: BusinessObject = None
    count: int = 0


@dataclass
class GenerateAlternateIdsResponse(TcBaseObj):
    """
    Holds the response for 'generateAlternateIds'
    
    :var inputIndexToAltId: Key is the index of the input, data is a list of alternate IDs generated.
    :var serviceData: Service data contains any partial errors and exceptions. No objects are created, deleted,
    modified or returned by this service.
    """
    inputIndexToAltId: IndexToAltId = None
    serviceData: ServiceData = None


@dataclass
class GetAllAttrMappingsResponse(TcBaseObj):
    """
    Contains the response for getAllAttrMappings operation.
    
    :var attrMappingInfos: A list of AttrMappingInfo
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information. CadAttrMappingDefinition objects are returned as plain objects.
    """
    attrMappingInfos: List[AttrMappingInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetAttrMappingsForDatasetTypeCriteria(TcBaseObj):
    """
    Input criteria for the data returned from the 'getAttrMappingsForDatasetType' operation.
    
    :var datasetTypeName: The type name of a dataset.  This input is required.
    :var itemTypeName: The type name of an item.  This input is optional.
    :var exact: The input flag indicating that the mappings to be returned are for a specific dataset type and item
    type combination when the value is true.
    """
    datasetTypeName: str = ''
    itemTypeName: str = ''
    exact: bool = False


@dataclass
class GetAttrMappingsForDatasetTypeOutput(TcBaseObj):
    """
    Contains the output data for the 'getAttrMappingsForDatasetType' operation.
    
    :var criteria: The dataset and item type criteria used to find the attribute mapping definitions.
    :var attrMappingInfos: The list of attribute mapping information that matches the 'criteria'.
    """
    criteria: GetAttrMappingsForDatasetTypeCriteria = None
    attrMappingInfos: List[AttrMappingInfo] = ()


@dataclass
class GetAttrMappingsForDatasetTypeResponse(TcBaseObj):
    """
    The response from the 'getAttrMappingsForDatasetType' operation.
    
    :var output: A list of input dataset and item type criteria and the found attribute mapping definitions.
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData' plain objects with
    'CadAttrMappingDefinition' objects and property descriptor LOV objects.
    """
    output: List[GetAttrMappingsForDatasetTypeOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetAvailableTypesResponse(TcBaseObj):
    """
    Holds the response for 'getAvailableTypes', which the map of input objects (class names) and for each input object,
    the object references of found Types (NULL, if none found), along with the 'ServiceData'.
    
    :var inputClassToTypes: Map where the key is the class type and the value is a list of strings representing the
    types available for the class.
    :var serviceData: Contains objects that are returned by the service, and error information. The actual type objects
    are returned as 'PlainObjects'. This service does not define any of its own errors, but will return any errors from
    the subsystem in the list of partial errors.
    """
    inputClassToTypes: ClassToTypesMap = None
    serviceData: ServiceData = None


@dataclass
class ItemInfo(TcBaseObj):
    """
    The ItemInfo struct represents all of the data necessary to construct the item object.
    The basic attributes that are required are passed as named elements in the struct.
    All other attributes are passed as name/value pairs in the AttributeInfo struct.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Item.
    
    :var item: Item object reference for update, can be null for creation
    :var itemId: ID for create, generated if null
    :var itemType: Type, default is Item if null
    :var name: Name, defaulted to id if null
    :var description: Description, can be null
    :var attrList: List of AttributeInfos
    :var extraObject: List of ExtraObjectInfos
    :var folder: Folder to attach Item to, if null then default used
    """
    item: Item = None
    itemId: str = ''
    itemType: str = ''
    name: str = ''
    description: str = ''
    attrList: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    folder: Folder = None


@dataclass
class ItemRevInfo(TcBaseObj):
    """
    The ItemRevInfo structure represents all of the data necessary to construct the item revision object.
    The basic attributes that are required are passed as named elements in the struct.
    All other attributes are passed as name/value pairs in the AttributeInfo struct.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Item
    Revision.
    
    :var itemRevision: ItemRevision object reference, null for creation, otherwise update
    :var revId: ID, if null then generated
    :var attrList: List of AttributeInfos
    :var extraObject: List ofr ExtraObjectInfos
    """
    itemRevision: ItemRevision = None
    revId: str = ''
    attrList: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()


@dataclass
class AttrMappingInfo(TcBaseObj):
    """
    Contains CadAttrMappingDefinition object reference and the PropDescriptor structure used in the response of get
    attribute mapping operations.
    
    :var cadAttrMappingDefinition: CadAttrMappingDefinition object reference
    :var propDesc: PropDescriptor structure containing property information for mapping definition property.
    """
    cadAttrMappingDefinition: CadAttrMappingDefinition = None
    propDesc: PropDescriptor = None


@dataclass
class MappedDatasetAttrProperty(TcBaseObj):
    """
    Contains the found resolved object and property name mapped to specific input values.
    
    :var attrTitle: The CadAttrMappingDefinition object title. This is generally the client application attribute
    display name.
    :var datasetTypeName: The Teamcenter defined type name of a dataset.
    :var itemTypeName: The Teamcenter defined type name of an item
    :var resolvedObject: Object reference of object holding mapped attribute value
    :var resolvedPropertyName: The property name of the mapped object holding the attribute value of interest resulting
    from evaluation of a dataset CAD attribute mapping definition.
    """
    attrTitle: str = ''
    datasetTypeName: str = ''
    itemTypeName: str = ''
    resolvedObject: BusinessObject = None
    resolvedPropertyName: str = ''


@dataclass
class NamedReferenceObjectInfo(TcBaseObj):
    """
    Contains information regarding named refernce type value, Object reference, Object type name and list of Attribute
    infos.
    
    :var clientId: Identifier defined by user to track the related object.
    :var object: Object reference of the object for update, null for create
    :var namedReferenceName: The Named Reference from the dataset to this object, required. NamedReference values  are
    defined for each Dataset type. The customer can add more values as needed. To get a current list of valid Named
    Reference values the programmer can either use BMIDE or can call the SOA Core service getDatasetTypeIno.
    :var typeName: Type of the object to be created. Required for object creation only.
    :var attrNameValuePairs: List of AttributeInfos.
    """
    clientId: str = ''
    object: BusinessObject = None
    namedReferenceName: str = ''
    typeName: str = ''
    attrNameValuePairs: List[AttributeInfo] = ()


@dataclass
class AttributeInfo(TcBaseObj):
    """
    This structure allows the caller to define or update named attributes.
    The name member represents the property name for the related object and the value is the value to set.
    
    :var name: Text for Attribute Name
    :var value: Text for Attribute Value
    """
    name: str = ''
    value: str = ''


@dataclass
class PartInfo(TcBaseObj):
    """
    The PartInfo struct is the main input to the createOrUpdateParts service.
    This struct refers to the Item, ItemRevision, and one or more Dataset structures used to create those objects.
    
    :var clientId: Identifier defined by user to track the related object.
    :var itemInput: Member of type ItemInfo
    :var itemRevInput: Member of type ItemRevInfo
    :var datasetInput: List of DatasetInfos
    """
    clientId: str = ''
    itemInput: ItemInfo = None
    itemRevInput: ItemRevInfo = None
    datasetInput: List[DatasetInfo] = ()


@dataclass
class PropDescriptor(TcBaseObj):
    """
    The PropDescriptor struct describes information about the Teamcenter property
    
    :var propName: Name of the property
    :var displayName: Display name of the property
    :var isModifiable: Specifies whether the property is modifiable
    :var attachedSpecifier: attachedSpecifier
    :var maxLength: maxLength
    :var interdependentProps: interdependentProps
    :var defaultValue: Default value for the property
    :var propValueType: Value type for the property in integer form: PROP_untyped (0) No Value
    PROP_char (1) Value is a single character
    PROP_date (2) Value is a date
    PROP_double (3) Value is a double
    PROP_float (4) Value is a float
    PROP_int (5) Value is an integer
    PROP_logical (6) Value is a logical
    PROP_short (7) Value is a short
    PROP_string (8) Value is a character string
    PROP_typed_reference (9) Value is a typed reference
    PROP_untyped_reference (10) Value is an untyped reference
    PROP_external_reference (11) Value is an external reference
    PROP_note (12) Value is a note
    PROP_typed_relation (13) Value is a typed relation
    PROP_untyped_relation (14) Value is an untyped relation
    :var propType: Type for the property in integer form: PROP_unknown (0) Property type is Unknown
    PROP_attribute (1)  Based on a POM Attribute (int, string, ...)
    PROP_reference (2)  Based on a POM Reference
    PROP_relation (3) Based on an ImanRelation
    PROP_compound (4) Based on a property from another Type
    PROP_runtime (5) Based on a computed value
    :var isDisplayable: isDisplayable
    :var isArray: Specifies whether the property is an array or single value
    :var lov: ListOfValues object attached to the property (if any)
    :var isRequired: Specifies whether the property is required
    :var isEnabled: Specifies whether the property is enabled
    """
    propName: str = ''
    displayName: str = ''
    isModifiable: bool = False
    attachedSpecifier: int = 0
    maxLength: int = 0
    interdependentProps: List[str] = ()
    defaultValue: str = ''
    propValueType: int = 0
    propType: int = 0
    isDisplayable: bool = False
    isArray: bool = False
    lov: ListOfValues = None
    isRequired: bool = False
    isEnabled: bool = False


@dataclass
class RelationAndTypesFilter2(TcBaseObj):
    """
    Structure contains relation name and vector of related object types of interest, which will used for filtering
    purpose.
    
    :var relationName: The name of the relation of interest.
    :var objectTypeNames: A list of related object types of interest.
    """
    relationName: str = ''
    objectTypeNames: List[str] = ()


@dataclass
class ResolveAttrMappingsForDatasetInfo(TcBaseObj):
    """
    Contains dataset, item revision and list of 'CadAttrMappingDefinition' object references to be used in the resolve.
    
    :var dataset: Dataset object reference for which to get mapped attribute values.
    :var itemRev: ItemRevision object reference, helps resolve ambiguity in the mapping traversal.
    :var cadAttrMappingDefinitions: List of 'CadAttrMappingDefinition' object references
    """
    dataset: Dataset = None
    itemRev: ItemRevision = None
    cadAttrMappingDefinitions: List[CadAttrMappingDefinition] = ()


@dataclass
class ResolveAttrMappingsForDatasetOutput(TcBaseObj):
    """
    Contains the output data for 'resolveAttrMappingsForDataset'.
    
    :var dataset: Dataset object reference for which mapped attribute property information is desired.
    :var mappedProperties: A list of 'MappedDatasetAttrProperty'
    """
    dataset: Dataset = None
    mappedProperties: List[MappedDatasetAttrProperty] = ()


@dataclass
class ResolveAttrMappingsForDatasetResponse(TcBaseObj):
    """
    Contains the response for 'resolveAttrMappingsForDataset'.
    
    :var output: A list of 'ResolveAttrMappingsForDatasetOutput'
    :var serviceData: 'ServiceData' contains any partial errors and exceptions.
    The objects holding the mapped attributes, resulting from successfully resolved mappings, are returned as plain
    objects.
    The mapped attribute properties are returned as 'ServiceData' properties.
    """
    output: List[ResolveAttrMappingsForDatasetOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CommitDatasetFileInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a dataset.
    
    :var dataset: Dateset object reference.
    :var createNewVersion: Flag to create new version ( TRUE ) or not ( FALSE ).
    :var datasetFileTicketInfos: A list of DatasetFileTicketInfos.
    """
    dataset: Dataset = None
    createNewVersion: bool = False
    datasetFileTicketInfos: List[DatasetFileTicketInfo] = ()


@dataclass
class CreateOrUpdatePartsOutput(TcBaseObj):
    """
    Intermediate level output structure for createOrUpdateParts operation.
    
    :var clientId: Identifier defined by user to track the related object.
    :var item: Item object reference of the created/updated item
    :var itemRev: ItemRevision object reference of the created/updated item revision
    :var datasetOutput: List of DatasetOutputs
    :var extraItemObjs: List of ExtraObjectOutputs for the item extra objects
    :var extraItemRevObjs: List of ExtraObjectOutputs for the item revision extra objects
    """
    clientId: str = ''
    item: Item = None
    itemRev: ItemRevision = None
    datasetOutput: List[DatasetOutput] = ()
    extraItemObjs: List[ExtraObjectOutput] = ()
    extraItemRevObjs: List[ExtraObjectOutput] = ()


@dataclass
class CreateOrUpdatePartsResponse(TcBaseObj):
    """
    Holds the response for createOrUpdateParts
    
    :var output: List of CreateOrUpdatePartsOutputs.
    :var serviceData: The SOA framework object containing objects that were created, deleted, or updated by the
    Service, plain objects, and error information.
    """
    output: List[CreateOrUpdatePartsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateRelationsInfo(TcBaseObj):
    """
    Contains input information for 'createdOrUpdateRelations' operation.
    
    :var clientId: Identifier defined by user to track the related object.
    :var relationType: Realtion of interest, required for create or update
    :var primaryObject: object reference of primary object of interest, required for create or update
    :var secondaryObject: object reference of secondary object of interest, required for create or update
    :var relation: ImanRelation object reference of existing relation, used for update if provided
    :var userData: object reference to an object that can be attached to the relation (optional)
    """
    clientId: str = ''
    relationType: str = ''
    primaryObject: BusinessObject = None
    secondaryObject: BusinessObject = None
    relation: ImanRelation = None
    userData: BusinessObject = None


@dataclass
class CreateOrUpdateRelationsOutput(TcBaseObj):
    """
    Contains the output response structure for 'createdOrUpdateRelations' operation.
    
    :var clientId: Identifier defined by user to track the related object.
    :var relation: ImanRelation object reference of relation created or updated
    """
    clientId: str = ''
    relation: ImanRelation = None


@dataclass
class CreateOrUpdateRelationsPref(TcBaseObj):
    """
    Contains a list of RelationAndTypesFilter structures for other side object filtering.
    
    :var info: A list of 'RelationAndTypesFilter2'
    """
    info: List[RelationAndTypesFilter2] = ()


@dataclass
class CreateOrUpdateRelationsResponse(TcBaseObj):
    """
    Contains the response for 'createOrUpdateRelations'
    
    :var output: A list of 'CreateOrUpdateRelationsOutput'
    :var serviceData: The SOA framework object containing objects that were created or updated by the service and error
    information. Errors are identified by the 'clientID' which is supplied in the input data.
    """
    output: List[CreateOrUpdateRelationsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class DatasetFileInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var fileName: Name of file to be uploaded.  Filename only, should not contain path to filename.
    :var namedReferencedName: Named Reference relation to file.
    :var isText: Flag to indicate if file is text ( TRUE ) or binary (FALSE ).
    :var allowReplace: Flag to indicate if file can be overwritten ( TRUE ) or not ( FALSE ).
    """
    clientId: str = ''
    fileName: str = ''
    namedReferencedName: str = ''
    isText: bool = False
    allowReplace: bool = False


@dataclass
class DatasetFileTicketInfo(TcBaseObj):
    """
    Holds the basic info for a file to be uploaded to a dataset.
    
    :var datasetFileInfo: Member of type DatasetFileInfo.
    :var ticket: ID of ticket.
    """
    datasetFileInfo: DatasetFileInfo = None
    ticket: str = ''


@dataclass
class DatasetInfo(TcBaseObj):
    """
    Contains all of the data necessary to construct the dataset object.
    The basic attributes that are required are passed as named elements in the structure.
    All other attributes are passed as name/value pairs in the AttributeInfo structure.
    The extraObject field allows for the creation of an object(s) that will be related to this newly created Dataset.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference for update, null for creation
    :var attrList: List of AttributeInfos for attributes
    :var mappingAttributes: List of AttributeInfos for mapped attributes. Mapped atributes are attributes that are
    applied to other objects. Refere to the ITK manual for a definition of attribute mapping.
    :var extraObject: List of ExtraObjectInfos
    :var datasetFileInfos: List of DatasetFileInfos
    :var namedReferenceObjectInfos: List of NamedReferenceObjectInfos
    :var name: Name attribute value
    :var description: Description attribute value
    :var type: Type attribute value
    :var id: ID attribute value
    :var datasetRev: Revision attribute value
    :var itemRevRelationName: Required input, may not be null, not defaulted
    :var createNewVersion: Flag to create new version ( TRUE ) or not (FALSE )
    :var namedReferencePreference: Preference name which holds the list of named references to delete from one Dataset
    version to the next.
    """
    clientId: str = ''
    dataset: Dataset = None
    attrList: List[AttributeInfo] = ()
    mappingAttributes: List[AttributeInfo] = ()
    extraObject: List[ExtraObjectInfo] = ()
    datasetFileInfos: List[DatasetFileInfo] = ()
    namedReferenceObjectInfos: List[NamedReferenceObjectInfo] = ()
    name: str = ''
    description: str = ''
    type: str = ''
    id: str = ''
    datasetRev: str = ''
    itemRevRelationName: str = ''
    createNewVersion: bool = False
    namedReferencePreference: str = ''


@dataclass
class DatasetOutput(TcBaseObj):
    """
    Structure contains created/updated dataset objects.
    
    :var clientId: Identifier defined by user to track the related object.
    :var dataset: Dataset object reference of the created/updated dataset
    :var commitInfo: List of CommitDatasetFileInfos
    :var extraObjs: List of ExtraObjectOutputs for the extra objects
    :var namedRefObjs: List of ExtraObjectOutputs for the named references
    """
    clientId: str = ''
    dataset: Dataset = None
    commitInfo: List[CommitDatasetFileInfo] = ()
    extraObjs: List[ExtraObjectOutput] = ()
    namedRefObjs: List[ExtraObjectOutput] = ()


"""
Map of integer index of the input to vector of string alternate id values (string, vector).
"""
IndexToAltId = Dict[int, List[str]]


"""
Map of string class names to vector of type string values (string, vector).
"""
ClassToTypesMap = Dict[str, List[str]]



from tcsoa.gen.Internal_AWS2._2018_05.services import GlobalAlternateService as imp0
from tcsoa.gen.Internal_AWS2._2017_06.services import EffectivityManagmentService as imp1
from tcsoa.gen.Internal_AWS2._2015_03.services import FullTextSearchService as imp2
from tcsoa.gen.Internal_AWS2._2012_10.services import DataManagementService as imp3
from tcsoa.gen.Internal_AWS2._2019_06.services import AdvancedSavedSearchService as imp4
from tcsoa.gen.Internal_AWS2._2016_12.services import AdvancedSearchService as imp5
from tcsoa.gen.Internal_AWS2._2017_06.services import RequirementsManagementService as imp6
from tcsoa.gen.Internal_AWS2._2012_10.services import FullTextSearchService as imp7
from tcsoa.gen.Internal_AWS2._2015_10.services import FullTextSearchService as imp8
from tcsoa.gen.Internal_AWS2._2018_05.services import FullTextSearchService as imp9
from tcsoa.gen.Internal_AWS2._2019_12.services import DataManagementService as imp10
from tcsoa.gen.Internal_AWS2._2020_05.services import UiConfigService as imp11
from tcsoa.gen.Internal_AWS2._2016_12.services import RequirementsManagementService as imp12
from tcsoa.gen.Internal_AWS2._2019_12.services import FinderService as imp13
from tcsoa.gen.Internal_AWS2._2018_12.services import FinderService as imp14
from tcsoa.gen.Internal_AWS2._2012_10.services import RequirementsManagementService as imp15
from tcsoa.gen.Internal_AWS2._2016_03.services import RequirementsManagementService as imp16
from tcsoa.gen.Internal_AWS2._2018_12.services import MultiSiteService as imp17
from tcsoa.gen.Internal_AWS2._2012_10.services import FinderService as imp18
from tcsoa.gen.Internal_AWS2._2017_12.services import DataManagementService as imp19
from tcsoa.gen.Internal_AWS2._2016_03.services import FinderService as imp20
from tcsoa.gen.Internal_AWS2._2018_05.services import DataManagementService as imp21
from tcsoa.gen.Internal_AWS2._2015_10.services import DataManagementService as imp22
from tcsoa.gen.Internal_AWS2._2016_12.services import DataManagementService as imp23
from tcsoa.gen.Internal_AWS2._2018_05.services import TCXMLService as imp24
from tcsoa.gen.Internal_AWS2._2014_11.services import RequirementsManagementService as imp25
from tcsoa.gen.Internal_AWS2._2020_05.services import FileMgmtService as imp26
from tcsoa.gen.Internal_AWS2._2019_06.services import RequirementsManagementService as imp27
from tcsoa.gen.Internal_AWS2._2012_10.services import OrganizationManagementService as imp28
from tcsoa.gen.Internal_AWS2._2013_12.services import OrganizationManagementService as imp29
from tcsoa.gen.Internal_AWS2._2017_12.services import FullTextSearchService as imp30
from tcsoa.gen.Internal_AWS2._2012_10.services import LOVService as imp31
from tcsoa.gen.Internal_AWS2._2016_04.services import DataManagementService as imp32
from tcsoa.gen.Internal_AWS2._2020_12.services import DataManagementService as imp33
from tcsoa.gen.Internal_AWS2._2016_03.services import UiConfigService as imp34
from tcsoa.gen.Internal_AWS2._2017_06.services import UiConfigService as imp35
from tcsoa.gen.Internal_AWS2._2012_10.services import StructureSearchService as imp36
from tcsoa.gen.Internal_AWS2._2020_05.services import DataManagementService as imp37
from tcsoa.gen.Internal_AWS2._2020_05.services import FullTextSearchService as imp38
from tcsoa.gen.Internal_AWS2._2015_03.services import DataManagementService as imp39
from tcsoa.gen.Internal_AWS2._2016_03.services import DataManagementService as imp40
from tcsoa.gen.Internal_AWS2._2017_06.services import DataManagementService as imp41
from tcsoa.gen.Internal_AWS2._2013_12.services import DataManagementService as imp42
from tcsoa.gen.Internal_AWS2._2012_10.services import WorkflowService as imp43
from tcsoa.gen.Internal_AWS2._2014_11.services import WorkflowService as imp44
from tcsoa.gen.Internal_AWS2._2018_05.services import WorkflowService as imp45
from tcsoa.gen.Internal_AWS2._2017_06.services import FullTextSearchService as imp46
from tcsoa.gen.Internal_AWS2._2018_05.services import FileMgmtService as imp47
from tcsoa.gen.Internal_AWS2._2019_06.services import DataManagementService as imp48
from tcsoa.gen.Internal_AWS2._2018_05.services import FinderService as imp49
from tcsoa.gen.Internal_AWS2._2016_12.services import FinderService as imp50
from tcsoa.gen.Internal_AWS2._2017_06.services import FinderService as imp51
from tcsoa.gen.Internal_AWS2._2017_12.services import FinderService as imp52
from tcsoa.gen.Internal_AWS2._2019_06.services import FinderService as imp53
from tcsoa.gen.Internal_AWS2._2013_12.services import WorkflowService as imp54
from tcsoa.base import TcService


class GlobalAlternateService(TcService):
    addAlternates = imp0.addAlternates
    removeAlternates = imp0.removeAlternates


class EffectivityManagmentService(TcService):
    addOrRemoveRelStatusEffectivities = imp1.addOrRemoveRelStatusEffectivities


class FullTextSearchService(TcService):
    cleanupScratchTable = imp2.cleanupScratchTable
    createFullTextSavedSearch = imp7.createFullTextSavedSearch
    createFullTextSavedSearch2 = imp8.createFullTextSavedSearch
    createFullTextSavedSearch3 = imp9.createFullTextSavedSearch
    deleteFullTextSavedSearch = imp8.deleteFullTextSavedSearch
    deleteIndexedIslands = imp2.deleteIndexedIslands
    deregisterApplicationIDs = imp2.deregisterApplicationIDs
    findFullTextSavedSearches = imp7.findFullTextSavedSearches
    getAMImpactedObjects = imp2.getAMImpactedObjects
    getAddedObjectsToUpdateIndex = imp7.getAddedObjectsToUpdateIndex
    getAddedObjectsToUpdateIndex1 = imp7.getAddedObjectsToUpdateIndex1
    getDatasetIndexableFilesInfo = imp7.getDatasetIndexableFilesInfo
    getDeletedObjectsToUpdateIndex = imp7.getDeletedObjectsToUpdateIndex
    getDeletedObjectsToUpdateIndex1 = imp7.getDeletedObjectsToUpdateIndex1
    getImpactedItemRevsForReIndex = imp7.getImpactedItemRevsForReIndex
    getIndexedObjects = imp2.getIndexedObjects
    getIndexedObjectsAndUpdate = imp30.getIndexedObjectsAndUpdate
    getModifiedObjectsToSync = imp2.getModifiedObjectsToSync
    getObjectsToIndex = imp7.getObjectsToIndex
    getPreFilters = imp7.getPreFilters
    getSearchSettings = imp38.getSearchSettings
    getSuggestions = imp7.getSuggestions
    identifyImpactedObjects = imp46.identifyImpactedObjects
    modifyFullTextSavedSearch = imp8.modifyFullTextSavedSearch
    modifyFullTextSavedSearch2 = imp9.modifyFullTextSavedSearch
    performFullTextSearch = imp7.performFullTextSearch
    queryAndUpdateSyncData = imp30.queryAndUpdateSyncData
    updateIndexIslandStatus = imp2.updateIndexIslandStatus
    updateIndexingStatus = imp7.updateIndexingStatus


class DataManagementService(TcService):
    clearHistory = imp3.clearHistory
    createIdDisplayRules = imp10.createIdDisplayRules
    getAvailableWorkspaces = imp19.getAvailableWorkspaces
    getChildren = imp3.getChildren
    getCurrentUserGateway = imp3.getCurrentUserGateway
    getCurrentUserGateway2 = imp21.getCurrentUserGateway2
    getDatasetTypesWithDefaultRelation = imp22.getDatasetTypesWithDefaultRelation
    getDeclarativeStyleSheets = imp23.getDeclarativeStyleSheets
    getDefaultRelation = imp23.getDefaultRelation
    getHistory = imp3.getHistory
    getIdContexts = imp10.getIdContexts
    getIdentifierTypes = imp10.getIdentifierTypes
    getInitialTableRowData = imp32.getInitialTableRowData
    getLocalizedProperties = imp33.getLocalizedProperties
    getRelatedObjsForConfiguredRevision = imp37.getRelatedObjsForConfiguredRevision
    getStyleSheet = imp39.getStyleSheet
    getStyleSheet2 = imp32.getStyleSheet
    getStyleSheet3 = imp40.getStyleSheet2
    getStyleSheet4 = imp41.getStyleSheet3
    getTCSessionAnalyticsInfo = imp19.getTCSessionAnalyticsInfo
    getTCSessionInfo = imp3.getTCSessionInfo
    getTCSessionInfo2 = imp42.getTCSessionInfo2
    getTCSessionInfo3 = imp42.getTCSessionInfo3
    getTableViewModelProperties = imp19.getTableViewModelProperties
    getUnprocessedXRT = imp40.getUnprocessedXRT
    getViewModelProperties = imp41.getViewModelProperties
    getViewModelProperties2 = imp19.getViewModelProperties2
    getViewerData = imp41.getViewerData
    loadDataForEditing = imp3.loadDataForEditing
    loadDataForEditing2 = imp32.loadDataForEditing
    loadViewModelForEditing = imp41.loadViewModelForEditing
    loadViewModelForEditing2 = imp19.loadViewModelForEditing2
    modifyFavorites = imp48.modifyFavorites
    pinObjects = imp21.pinObjects
    saveEdit = imp3.saveEdit
    saveEditAndSubmitToWorkflow = imp3.saveEditAndSubmitToWorkflow
    saveEditAndSubmitToWorkflow2 = imp32.saveEditAndSubmitToWorkflow
    saveEditAndSubmitToWorkflow3 = imp23.saveEditAndSubmitToWorkflow
    saveViewModelEditAndSubmitWorkflow = imp41.saveViewModelEditAndSubmitWorkflow
    saveViewModelEditAndSubmitWorkflow2 = imp21.saveViewModelEditAndSubmitWorkflow2
    saveXRT = imp40.saveXRT
    setLocalizedProperties = imp33.setLocalizedProperties
    unpinObjects = imp21.unpinObjects
    updateHistory = imp3.updateHistory
    updateTiles = imp21.updateTiles


class AdvancedSavedSearchService(TcService):
    createAdvancedSavedSearch = imp4.createAdvancedSavedSearch
    updateAdvancedSavedSearch = imp4.updateAdvancedSavedSearch


class AdvancedSearchService(TcService):
    createAdvancedSearchInput = imp5.createAdvancedSearchInput
    getSelectedQueryCriteria = imp5.getSelectedQueryCriteria


class RequirementsManagementService(TcService):
    createBaseline = imp6.createBaseline
    createBaselineAsync = imp6.createBaselineAsync
    exportAsync = imp12.exportAsync
    exportAsync2 = imp6.exportAsync2
    exportToApplication = imp15.exportToApplication
    exportToApplication2 = imp16.exportToApplication
    exportToApplication3 = imp12.exportToApplication2
    exportToApplication4 = imp6.exportToApplication3
    getExportTemplates = imp25.getExportTemplates
    getFullTextVersionInfo = imp27.getFullTextVersionInfo
    setRichContent = imp15.setRichContent
    setRichContent2 = imp12.setRichContent2


class UiConfigService(TcService):
    createNamedColumnConfig = imp11.createNamedColumnConfig
    deleteNamedColumnConfig = imp11.deleteNamedColumnConfig
    getNamedColumnConfigs = imp11.getNamedColumnConfigs
    getOrResetUIColumnConfigs = imp34.getOrResetUIColumnConfigs
    getOrResetUIColumnConfigs2 = imp35.getOrResetUIColumnConfigs2
    getVisibleCommands = imp34.getVisibleCommands
    loadNamedColumnConfig = imp11.loadNamedColumnConfig
    saveNamedColumnConfig = imp11.saveNamedColumnConfig
    saveUIColumnConfigs = imp34.saveUIColumnConfigs


class FinderService(TcService):
    exportObjectsToFile = imp13.exportObjectsToFile
    exportSearchResults = imp14.exportSearchResults
    findObjectsByClassAndAttributes2 = imp18.findObjectsByClassAndAttributes2
    findUsersTasks = imp18.findUsersTasks
    getClassificationProps = imp20.getClassificationProps
    getFilterValues = imp13.getFilterValues
    performFacetSearch = imp49.performFacetSearch
    performSearch = imp20.performSearch
    performSearch2 = imp50.performSearch2
    performSearchViewModel = imp51.performSearchViewModel
    performSearchViewModel2 = imp52.performSearchViewModel2
    performSearchViewModel3 = imp49.performSearchViewModel3
    performSearchViewModel4 = imp53.performSearchViewModel4


class MultiSiteService(TcService):
    fetchODSRecords = imp17.fetchODSRecords


class TCXMLService(TcService):
    getDiagnosticInfoForAcctTables = imp24.getDiagnosticInfoForAcctTables
    installDBTriggersForDataSync = imp24.installDBTriggersForDataSync
    removeDBTriggersForDataSync = imp24.removeDBTriggersForDataSync


class FileMgmtService(TcService):
    getFileNamesWithTicketInfo = imp26.getFileNamesWithTicketInfo
    loadPlmdTicketForReplace = imp47.loadPlmdTicketForReplace


class OrganizationManagementService(TcService):
    getGroupMembership = imp28.getGroupMembership
    getGroupMembership2 = imp29.getGroupMembership2


class LOVService(TcService):
    getInitialLOVValues = imp31.getInitialLOVValues
    getNextLOVValues = imp31.getNextLOVValues
    validateLOVValueSelections = imp31.validateLOVValueSelections


class StructureSearchService(TcService):
    getParentsWhereUsed = imp36.getParentsWhereUsed
    getProductsWhereUsed = imp36.getProductsWhereUsed


class WorkflowService(TcService):
    getTaskResults = imp43.getTaskResults
    getWorkflowGraph = imp44.getWorkflowGraph
    getWorkflowGraphLegend = imp44.getWorkflowGraphLegend
    getWorkflowTaskViewModel = imp45.getWorkflowTaskViewModel
    performTaskSearch = imp54.performTaskSearch



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, Awp0XRTObjectSet, POM_object, Dataset
from tcsoa.gen.Internal.AWS2._2013_12.DataManagement import GetStyleSheetDatasetInfo, GetStyleSheetClassificationData
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetStyleSheetOutput(TcBaseObj):
    """
    Output from getStyleSheet operation.
    
    :var processedPage: The name of the page that was processed. If the operation was invoked with processEntireXRT set
    to true, the this field is empty. If the operation was invoked with processEntireXRT set to false, this this field
    is populated with the page that was processed. Typically the value of the processedPage would be the same as the
    targetPage, except when the targetPage value is empty or not found in the stylesheet. In that case, then the first
    page in the stylesheet is processed.
    :var datasetInfo: The information about the stylesheet.
    :var context: Contains the information about what this stylesheet was registered to.
    :var imageTypeTicketMap: Map (string/string) where the key is the image resolution type from the stylesheet and
    value is the FMS ticket of the image for that resolution type.When resolution is not specified in the stylesheet or
    is invalid, FMS ticket mentioned in the default key will be populated.If the styesheet being rendered is SUMMARY,
    default value will be FMS ticket of the High resolution derived Dataset else default value would be FMS ticket of
    the Medium resolution derived Dataset.If derived Dataset does not exist,default value will be FMS ticket of the
    original Dataset.
    :var objectSetMap: A Map (string/Teamcenter::Awp0XRTObjectSet) where key is the source field from the stylesheet
    and the value is the data to present inside that objectSet in the application.
    :var tablePropObjectMap: A map(string/vector of POM_object) where key is the name of the table property and the
    value is a vector of Table Row objects which represents the table row data for that property.
    :var localeMap: Map (string/string) where the key is the text, title, or titleKey string from the stylesheet and
    the value is the localized string.
    :var jtFileMap: Map (BusinessObject/string) where the key is the business object to render, and the value is the
    thumbnail ticket for the image to download.
    :var visiblePages: The visible pages in the XRT. Each index in the array indicates whether that page can be
    displayed.
    :var classificationData: The classification data for the object to be rendered.
    :var objectToRender: The object to render.  It could be the input object itself in the case of viewing a
    stylesheet, or the revise descriptor object in the case of the revise operation.
    :var patternMap: Map containing the patterns used for auto-assignable properties. The key is the property name, and
    the value is the patterns to use. Only auto assignable properties exist in the map.
    """
    processedPage: str = ''
    datasetInfo: GetStyleSheetDatasetInfo = None
    context: StylesheetContext = None
    imageTypeTicketMap: StringMap6 = None
    objectSetMap: ObjectSetMap5 = None
    tablePropObjectMap: TablePropObjectMap2 = None
    localeMap: StringMap6 = None
    jtFileMap: ThumbnailMap5 = None
    visiblePages: List[bool] = ()
    classificationData: List[GetStyleSheetClassificationData] = ()
    objectToRender: BusinessObject = None
    patternMap: PatternMap2 = None


@dataclass
class GetStyleSheetResponse(TcBaseObj):
    """
    Response sent to client from the getStyleSheet operation.
    
    :var output: The vector of output information. One for each input object.
    :var serviceData: The SOA service data.
    """
    output: List[GetStyleSheetOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetUnprocessedXRTResponse(TcBaseObj):
    """
    Returns unprocessed XRT information.
    
    :var serviceData: The SOA service data.
    :var dsInfo: The stylesheet dataset information.
    """
    serviceData: ServiceData = None
    dsInfo: DSInfo = None


@dataclass
class InjectDSInfo(TcBaseObj):
    """
    Represents injected dataset (snippet) information.
    
    :var dataset: The dataset object.
    :var datasetName: The dataset name.
    :var xml: The contents of the XRT file.
    """
    dataset: Dataset = None
    datasetName: str = ''
    xml: str = ''


@dataclass
class PatternInfo2(TcBaseObj):
    """
    Structure defining pattern information.
    
    :var patterns: The naming rule patterns.
    :var preferredPattern: The preferred pattern to use. Typically would be the last used pattern.
    :var condition: The condition.
    """
    patterns: List[str] = ()
    preferredPattern: str = ''
    condition: str = ''


@dataclass
class StylesheetContext(TcBaseObj):
    """
    Defines what context the stylesheet was registered to.
    
    :var client: The client for which this is applicable. Ex: "AWC".
    :var type: The Teamcenter data type.
    :var location: The location in the client, if applicable. Example: showObjectLocation
    :var sublocation: The sublocation in the client, if applicable. Example: objectNavigationSubLocation
    :var stylesheetType: The stylesheet type.  Valid values are CREATE, SAVEAS, REVISE, SUMMARY and INFO.
    :var preferenceLocation: The preference location. Valid values are USER, ROLE, GROUP, SITE and ALL.
    :var datasetName: The dataset name to which this Dataset was registered.
    """
    client: str = ''
    type: str = ''
    location: str = ''
    sublocation: str = ''
    stylesheetType: str = ''
    preferenceLocation: str = ''
    datasetName: str = ''


@dataclass
class DSInfo(TcBaseObj):
    """
    The stylesheet information.
    
    :var datasetObject: The dataset object.
    :var stylesheetContext: The context to which this stylesheet was registered.
    :var xrt: The content of the XRT.
    :var injectedByPreference: Map(string, list of InjectDSInfo) where the key is a preference name and the value is  a
    list of InjectDSInfo objects which contain the Dataset object, name and contents of the XRT.
    :var injectedByDatasetName: Map(string, InjectDSInfo) where the key is a Dataset name and the value is an
    InjectDSInfo object which contains the Dataset object, name, and contents of the XRT.
    """
    datasetObject: Dataset = None
    stylesheetContext: StylesheetContext = None
    xrt: str = ''
    injectedByPreference: InjectedByPreferenceMap = None
    injectedByDatasetName: InjectedByNameMap = None


"""
Map of dataset name to its DSInfo object.
"""
InjectedByNameMap = Dict[str, InjectDSInfo]


"""
Map of preferenceName to vector of InjectDSInfo.
"""
InjectedByPreferenceMap = Dict[str, List[InjectDSInfo]]


"""
Maps the object set source in the xml rendering style sheet to the business objects.
"""
ObjectSetMap5 = Dict[str, Awp0XRTObjectSet]


"""
Map of string to vector of strings. This map is used to return the auto assignable property name patterns to the client. The key is the property name and the value is the list of patterns.
"""
PatternMap2 = Dict[str, PatternInfo2]


"""
String map.
"""
StringMap6 = Dict[str, str]


"""
Maps the table property in the xml rendering style sheet to the Table Row objects.
Note: This output data structure is not available for consumption in Teamcenter 10.1.x Release stream.
"""
TablePropObjectMap2 = Dict[str, List[POM_object]]


"""
Maps a business object to a thumbnail file ticket.
"""
ThumbnailMap5 = Dict[BusinessObject, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SearchOrders(TcBaseObj):
    """
    This is to provide the relation search order and dataset type search order preferences
    
    :var relationSearchOrder: The relation search order to look for to get the thumbnail file.
    :var datasetTypeSearchOrder: The dataset types order to look for to get the thumbnail file in a particular relation.
    """
    relationSearchOrder: List[str] = ()
    datasetTypeSearchOrder: List[str] = ()


@dataclass
class ThumbnailFileTicketsResponse(TcBaseObj):
    """
    Return type consisiting of list of thumbnail file tickets in the order of the input list of business objects.
    
    :var thumbnailFileTickets: List of thumbnail file tickets in the same order as input business objects.
    :var serviceData: This will contain the business objects for which thumbnail file ticket is not generated in the
    form of partial errors.
    """
    thumbnailFileTickets: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateThumbnailInputs(TcBaseObj):
    """
    This structure stores user inputs for updating thumbnail.
    
    :var isNone: True value is to specify no source for thumbnail. Type icon will be used as thumbnail.
    :var isAutoSelect: True value lets system determine source for thumbnail.
    :var thumbnailSource: User specified source for thumbnail. This object should be of type Dataset.
    """
    isNone: bool = False
    isAutoSelect: bool = False
    thumbnailSource: BusinessObject = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetValidoverlayBomlineResponse(TcBaseObj):
    """
    The 'GetValidoverlayBomlineResponse' structure contains the configured and unconfigured BOMLine objects after the
    VOO operation and the 'ServiceData'.
    
    :var validoverlayBomlines: vector of 'ConfigbomlinesInfoForValidoverlay' objects
    :var serviceData: An object of 'ServiceData' which contains any PartialErrors if occurred during execution of the
    VOO or effectivity date filtering
    """
    validoverlayBomlines: List[ConfigbomlinesInfoForValidoverlay] = ()
    serviceData: ServiceData = None


@dataclass
class BomlinesInfoForValidoverlay(TcBaseObj):
    """
    The 'BomlinesInfoForValidoverlay' data structure contains background BOMLine objects on which VOO needs to be
    applied along with other parameters to configure the VOO operation
    
    :var showUnconfiguredVariant: Flag to specify whether unconfigured BOMLine objects need to be displayed
    :var showUnconfiguredBydate: Flag to specify whether the effectivity date based filtering needs to be applied. true
    value means it should be skipped.
    :var variantRuleTag: List of VariantRule tag, for each variant rule
    :var backgrndBomlines: List of background BOMLine objects for which VOO needs to be applied.
    """
    showUnconfiguredVariant: bool = False
    showUnconfiguredBydate: bool = False
    variantRuleTag: List[BusinessObject] = ()
    backgrndBomlines: List[BusinessObject] = ()


@dataclass
class ConfigbomlinesInfoForValidoverlay(TcBaseObj):
    """
    The 'ConfigbomlinesInfoForValidoverlay' structure contains lists of configured and unconfigured BOMLine objects,
    after the VOO filtering operation is completed.
    
    :var backgrndBomlines: List of configured BOMLine objects
    :var unconfiguredBomlines: List of unconfigured BOMLine objects post the filtering
    """
    backgrndBomlines: List[BusinessObject] = ()
    unconfiguredBomlines: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ImanFile
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CommitReplacedFileInfo(TcBaseObj):
    """
    The FMS write ticket with which the replacement data file was uploaded to the transient volume.  This is the FMS
    write ticket issued by the 'FileManagementService::getTransientFileTicketsForUpload'() operation.  The
    'FileManagementService::commitReplacedFiles'() operation uses this ticket to locate the data file in the transient
    volume.
    
    :var replaceFileTicket: File write ticket.
    :var newOriginalFileName: If specified, this is a name for the new file.  If this filename is an empty string, the
    existing filename in the volume is used.
    :var imanFile: This is the ImanFile object with which the transient volume data file should be associated.
    """
    replaceFileTicket: str = ''
    newOriginalFileName: str = ''
    imanFile: ImanFile = None
